"""Test automatically write out of restart files"""
from __future__ import print_function
import os
from gpaw import GPAW
from ase import Atom, Atoms
from gpaw.test import equal

##endings = ['nc']
endings = ['gpw']
try:
    import _gpaw_hdf5
    endings.append('hdf5')
except ImportError:
    pass

for ending in endings:
    restart_wf = 'gpaw-restart-wf.' + ending
    # H2
    H = Atoms([Atom('H', (0,0,0)), Atom('H', (0,0,1))])
    H.center(vacuum=2.0)

    if 1:
        calc = GPAW(nbands=2,
                    convergence={'eigenstates': 0.001,
                                 'energy': 0.1,
                                 'density': 0.1})
        H.set_calculator(calc)
        H.get_potential_energy()
        calc.write(restart_wf, 'all')

        # refine the result directly
        calc.set(convergence={'energy': 0.00001})
        Edirect = H.get_potential_energy()

    # refine the result after reading from a file
    H = GPAW(restart_wf, convergence={'energy': 0.00001}).get_atoms()
    Erestart = H.get_potential_energy()

    print(Edirect, Erestart)
    # Note: the different density mixing introduces small differences 
    equal(Edirect, Erestart, 4e-5)



from __future__ import print_function
import numpy as np
from math import pi, cos, sin
from ase import Atom, Atoms
# from ase.parallel import rank, barrier
from gpaw import GPAW, PoissonSolver
from gpaw.xas import XAS
from gpaw.test import equal, gen

# Generate setup for oxygen with half a core-hole:
gen('O', name='hch1s', corehole=(1, 0, 0.5))

a = 5.0
d = 0.9575
t = pi / 180 * 104.51
H2O = Atoms([Atom('O', (0, 0, 0)),
             Atom('H', (d, 0, 0)),
             Atom('H', (d * cos(t), d * sin(t), 0))],
            cell=(a, a, a), pbc=False)
H2O.center()
calc = GPAW(nbands=10, h=0.2, setups={'O': 'hch1s'},
            poissonsolver=PoissonSolver(use_charge_center=True))
H2O.set_calculator(calc)
e = H2O.get_potential_energy()
niter = calc.get_number_of_iterations()

import gpaw.mpi as mpi

if mpi.size == 1:
    xas = XAS(calc)
    x, y = xas.get_spectra()
    e1_n = xas.eps_n
    de1 = e1_n[1] - e1_n[0]

calc.write('h2o-xas.gpw')

if mpi.size == 1:
    calc = GPAW('h2o-xas.gpw', txt=None,
                poissonsolver=PoissonSolver(use_charge_center=True))
    calc.initialize()
    xas = XAS(calc)
    x, y = xas.get_spectra()
    e2_n = xas.eps_n
    w_n = np.sum(xas.sigma_cn.real**2, axis=0)
    de2 = e2_n[1] - e2_n[0]

    print(de2)
    print(de2 - 2.0705)
    assert abs(de2 - 2.0705) < 0.001
    print(w_n[1] / w_n[0])
    assert abs(w_n[1] / w_n[0] - 2.22) < 0.01

    if mpi.size == 1:
        assert de1 == de2


if 0:
    import pylab as p
    p.plot(x, y[0])
    p.plot(x, sum(y))
    p.show()

print(e, niter)
energy_tolerance = 0.00009
niter_tolerance = 0
equal(e, -17.9772, energy_tolerance)



import numpy as np

from gpaw.xc.hybrid import HybridXCBase


class ForceCalculator:
    def __init__(self, timer):
        self.timer = timer
        self.reset()
        
    def reset(self):
        self.F_av = None

    def calculate(self, wfs, dens, ham):
        """Return the atomic forces."""

        assert not isinstance(ham.xc, HybridXCBase)

        if self.F_av is not None:
            return self.F_av

        self.timer.start('Force calculation')

        natoms = len(wfs.setups)

        # Force from projector functions (and basis set):
        F_wfs_av = np.zeros((natoms, 3))
        wfs.calculate_forces(ham, F_wfs_av)
        wfs.gd.comm.sum(F_wfs_av, 0)
                
        F_ham_av = np.zeros((natoms, 3))

        try:
            # ODD functionals need force corrections for each spin
            correction = ham.xc.setup_force_corrections
        except AttributeError:
            pass
        else:
            correction(F_ham_av)

        ham.calculate_forces(dens, F_ham_av)

        F_av = F_ham_av + F_wfs_av
        wfs.world.broadcast(F_av, 0)

        self.F_av = wfs.kd.symmetry.symmetrize_forces(F_av)
        self.timer.stop('Force calculation')
        return self.F_av



from gpaw.transport.selfenergy import LeadSelfEnergy
from gpaw.transport.tools import get_matrix_index, aa1d, aa2d, sum_by_unit, \
                                dot, fermidistribution, eig_states_norm, \
                                find, get_atom_indices, dagger, \
                                write, gather_ndarray_dict
from gpaw.transport.sparse_matrix import Tp_Sparse_Matrix
from gpaw.mpi import world
from gpaw import GPAW, Mixer, MixerDif, PoissonSolver
from ase.units import Hartree, Bohr
from gpaw.utilities.memory import maxrss
import numpy as np
import copy
import pickle
import os

class Transport_Analysor:
    def __init__(self, tp, restart=False):
        self.restart = restart
        self.data = {}
        self.ele_steps = []
        self.bias_steps = []
        self.ion_steps = []
        self.n_ele_step = 0
        self.n_bias_step = tp.n_bias_step
        self.n_ion_step = tp.n_ion_step
        self.matrix_foot_print = False
        self.reset = False
        self.scattering_states_initialized = False
        self.initialize(tp)
       
    def initialize(self, tp):
        kw = tp.analysis_parameters
        if self.restart:
            self.initialize_selfenergy_and_green_function(tp)
        else:
            self.selfenergies = tp.selfenergies
        p = self.set_default_analysis_parameters()
        for key in kw:
            if key in ['energies', 'lead_pairs', 'dos_project_atoms',
                       'project_equal_atoms', 'project_molecular_levels',
                       'isolate_atoms', 'dos_project_orbital',
                       'trans_project_orbital', 'eig_trans_channel_energies',
                        'eig_trans_channel_num', 'dos_realspace_energies']:
                p[key] = kw[key]
        for key in p:
            vars(self)[key] = p[key]

        ef = tp.lead_fermi[0]
        path = tp.contour.get_plot_path()
        self.energies = path.energies
        self.my_energies = path.my_energies
        self.weights = path.weights
        self.my_weights = path.my_weights
        self.nids = path.nids
        self.my_nids = path.my_nids
        
        for variable in [self.eig_trans_channel_energies,
                         self.dos_realspace_energies]:
            if variable is not None:
                variable = np.array(variable) + ef
        ecomm = tp.contour.comm
        
        if self.eig_trans_channel_energies is not None:
            self.my_eig_trans_channel_energies = np.array_split(
                self.eig_trans_channel_energies, ecomm.size)[ecomm.rank]
            self.my_eig_trans_channel_nids = np.array_split(
                 np.arange(len(self.eig_trans_channel_energies)),
                 ecomm.size)[ecomm.rank]
        
        if self.dos_realspace_energies is not None:
            self.my_dos_realspace_energies = np.array_split(
                 self.dos_realspace_energies, ecomm.size)[ecomm.rank]
            self.my_dos_realspace_nids = np.array_split(
                np.arange(len(self.dos_realspace_energies)),
                ecomm.size)[ecomm.rank]

        setups = tp.inner_setups
        self.project_atoms_in_device = self.project_equal_atoms[0]
        self.project_atoms_in_molecule = self.project_equal_atoms[1]
        self.project_basis_in_device = get_atom_indices(
                                  self.project_atoms_in_device, setups)
        if self.isolate_atoms is not None:
            self.calculate_isolate_molecular_levels(tp)
        self.overhead_data_saved = False
        if world.rank == 0:
            if not os.access('analysis_data', os.F_OK):
                os.mkdir('analysis_data')
            if not os.access('analysis_data/ionic_step_0', os.F_OK):
                os.mkdir('analysis_data/ionic_step_0')
        world.barrier()
        self.data = {}

    def save_overhead_data(self, tp):
        tp.log('-> save_overhead_data()')
        contour_parameters = {}
        cp = contour_parameters
        cp['neintmethod'] = tp.neintmethod
        cp['neintstep'] = tp.neintstep
        cp['lead_fermi'] = tp.lead_fermi
        cp['kt'] = tp.occupations.width * Hartree
        if not tp.non_sc:
            cp['eqinttol'] = tp.contour.eqinttol
        
        basis_information = {}
        bi = basis_information
        bi['orbital_indices'] = tp.orbital_indices
        bi['lead_orbital_indices'] = tp.lead_orbital_indices
        tp.log('   tp.hsd.S[0] : {0}'.format(tp.hsd.S[0]))
        bi['ll_index'] = tp.hsd.S[0].ll_index
        bi['ex_ll_index'] = tp.hsd.S[0].ex_ll_index
          
        atoms = tp.atoms.copy()
        if world.rank == 0:
            fd = open('analysis_overhead', 'wb')
            tp.log('   cPickle.dump()')
            pickle.dump((atoms, basis_information, contour_parameters), fd, 2)
            fd.close()
        tp.log('<- save_overhead_data()')
           
    def set_default_analysis_parameters(self):
        p = {}
        p['energies'] = np.linspace(-5., 5., 201)
        p['lead_pairs'] = [[0,1]]
        p['dos_project_atoms'] = None
        p['project_molecular_levels'] = None
        p['project_equal_atoms'] = [[], []]
        p['isolate_atoms'] = None
        p['dos_project_orbital'] = None
        p['trans_project_orbital'] = None
        p['eig_trans_channel_energies'] = None
        p['eig_trans_channel_num'] = 0
        p['dos_realspace_energies'] = None
        return p
                
    def initialize_selfenergy_and_green_function(self, tp):
        self.selfenergies = []
        if tp.use_lead:
            for i in range(tp.lead_num):
                self.selfenergies.append(LeadSelfEnergy(tp.lead_hsd[i],
                                                     tp.lead_couple_hsd[i]))
    
                self.selfenergies[i].set_bias(tp.bias[i])
            
    def reset_selfenergy(self, tp, s, k):
        for i in range(tp.lead_num):
            self.selfenergies[i].s = s
            self.selfenergies[i].pk = k
    
    def reset_green_function(self, tp, s, k):
        tp.hsd.s = s
        tp.hsd.pk = k
      
    def calculate_green_function_of_k_point(self, tp, s, k, energy, sigma,
                                                                 full=False):
        self.reset_green_function(tp, s, k)
        return tp.hsd.calculate_eq_green_function(energy, sigma,
                                                       ex=False, full=full)
 
    def calculate_sigma(self, tp, s, k, energy, nid_flag=None):
        sigma = []
        for i in range(tp.lead_num):
            sigma.append(self.selfenergies[i](energy, nid_flag))
        return sigma

    def get_gamma(self, tp, sigma):
        gamma = []
        for i in range(tp.lead_num):
            gamma.append(1.j * (sigma[i].recover() -
                                                sigma[i].recover().T.conj()))
        return gamma
        
    def calculate_transmission(self, tp, s, k, energy, nid_flag=None):
        tp.log('-> calculate_transmission()')
        self.reset_selfenergy(tp, s, k)
        self.reset_green_function(tp, s, k)
        tp.log('  calculate_sigma()')
        sigma = self.calculate_sigma(tp, s, k, energy, nid_flag)
        tp.log('  calculate_gamma()')
        gamma = self.get_gamma(tp, sigma)
        trans_coff = []
        for i, lead_pair in enumerate(self.lead_pairs):
            l1, l2 = lead_pair
            if i == 0:
                tp.log('  tp.hsd.abstract_sub_green_matrix()')
                gr_sub, inv_mat = tp.hsd.abstract_sub_green_matrix(
                                                    energy, sigma, l1, l2)
            else:
                tp.log('  tp.hsd.abstract_sub_green_matrix()')
                gr_sub = tp.hsd.abstract_sub_green_matrix(energy,
                                                    sigma, l1, l2, inv_mat)
            transmission =  dot(dot(gamma[l1], gr_sub),
                                              dot(gamma[l2], gr_sub.T.conj()))
            trans_coff.append(np.real(np.trace(transmission)))
        trans_coff = np.array(trans_coff)
        tp.log('<- calculate_transmission()')
        return trans_coff
    
    def calculate_dos(self, tp, s, k, energy, nid_flag=None):
        self.reset_selfenergy(tp, s, k)
        self.reset_green_function(tp, s, k)
        sigma = self.calculate_sigma(tp, s, k, energy, nid_flag)
        gr = self.calculate_green_function_of_k_point(tp, s, k, energy, sigma)
        dos = - np.imag(np.diag(dot(gr, tp.hsd.S[k].recover()))) / np.pi
        return dos
    
    def calculate_eigen_transport_channel(self, tp, s, q, energy):
        t, vc, k, vl = self.central_scattering_states(tp, energy, s, q)
        weights = []
        velocities = []
        vectors = []
        for i in range(tp.lead_num):
            weights.append([])
            velocities.append([])
            vectors.append([])
            for j in range(tp.lead_num):
                zeta = t[i][j]
                zeta2 = np.dot(zeta.T.conj(), zeta)
                if np.product(zeta2.shape) != 0:
                    w, v = np.linalg.eig(zeta2)
                    weights[i].append(w)
                    velocities[i].append(v)
                    vectors[i].append(np.dot(vc[i], v))
        weights = np.array(weights)
        velocities = np.array(velocities)
        vectors = np.array(vectors)
        return weights, velocities, vectors
  
    def calculate_charge_distribution(self, tp, s, q):
        d_mm = tp.hsd.D[s][q].recover(True)
        s_mm = tp.hsd.S[q].recover(True)
        q_mm = np.dot(d_mm, s_mm)
        return np.diag(q_mm).real

    def calculate_isolate_molecular_levels(self, tp):
        atoms = self.isolate_atoms
        atoms.pbc = True
        p = tp.gpw_kwargs.copy()
        p['nbands'] = None
        if 'mixer' in p:
            if not tp.spinpol:
                p['mixer'] = Mixer(0.1, 5, weight=100.0)
            else:
                p['mixer'] = MixerDif(0.1, 5, weight=100.0)
        p['poissonsolver'] = PoissonSolver(nn=2)
        if isinstance(p['basis'], dict) and len(p['basis']) == len(tp.atoms):
            p['basis'] = 'dzp'
            raise Warning('the dict basis is not surpported in isolate atoms')
        if 'txt' in p and p['txt'] != '-':
            p['txt'] = 'isolate_' + p['txt']
        atoms.set_calculator(GPAW(**p))
        atoms.get_potential_energy()
        setups = atoms.calc.wfs.setups
        self.project_basis_in_molecule = get_atom_indices(
                                      self.project_atoms_in_molecule, setups)
        kpt = atoms.calc.wfs.kpt_u[0]
        s_mm = atoms.calc.wfs.S_qMM[0]
        c_nm = eig_states_norm(kpt.C_nM, s_mm)
        self.isolate_eigen_values = kpt.eps_n
        self.isolate_eigen_vectors = c_nm
        self.isolate_s_mm = s_mm

    def calculate_project_transmission(self, tp, s, q, energy):
        eps_n, c_nm, s_mm = self.isolate_eigen_values, \
                                 self.isolate_eigen_vectors, self.isolate_s_mm
        nl = tp.lead_num
        T0 = np.zeros(c_nm.shape[0])
        ind1 = self.project_basis_in_molecule
        ind2 = self.project_basis_in_device
        t, vc, k, vl = self.central_scattering_states(tp, energy, s, q)
        project_transmission = []
        for j in range(nl):
            project_transmission.append([])
            for k in range(nl):
                vs = vc[k][ind2]
                vm = np.dot(np.dot(c_nm.T.conj(), s_mm)[:, ind1], vs)
                t0 = t[j][k]
                if len(t0) > 0:
                    pt = vm * vm.conj() * \
                             np.diag(np.dot(t0.T.conj(), t0)) \
                                  / np.diag(np.dot(vm.T.conj(), vm))
                else:
                    pt = T0
                project_transmission[j].append(pt)
        project_transmission = np.array(project_transmission)
        return project_transmission

    def calculate_realspace_wave_functions(self, tp, C_nm, q):
        #nl number of molecular levels
        wfs = tp.wfs
        nao = wfs.setups.nao
        nb, nl = C_nm.shape
        if wfs.dtype == float:
            C_nm = C_nm.real.copy()
        #extended_C_nm = np.zeros([nl, nao], wfs.dtype)
        total_psi_g = []
        for i in range(nl):
            psi_g = tp.gd.zeros(nl, dtype=wfs.dtype)
            c_nm = C_nm.reshape(1, -1)
            psi_g = psi_g.reshape(1, -1)
            wfs.basis_functions.lcao_to_grid(c_nm, psi_g, q)
            psi_g.shape = tp.gd.n_c
            global_psi_g = tp.gd.collect(psi_g / Bohr**1.5)
            total_psi_g.append(global_psi_g)
        return np.array(total_psi_g)

    def get_left_channels(self, tp, energy, s, k):
        # to get the left scattering channel from lead to scattering region
        sigma = self.calculate_sigma(tp, s, k, energy)
        g_s_ii = self.calculate_green_function_of_k_point(tp, s, k, energy,
                                                          sigma, full=True)
        nb = g_s_ii.shape[-1]
        dtype = g_s_ii.dtype
        lambda_l_ii = np.zeros([nb, nb], dtype)
        lambda_r_ii = np.zeros([nb, nb], dtype)
        ind = get_matrix_index(tp.hsd.S[0].ll_index[0][-1])
        lambda_l_ii[ind.T, ind] = 1.j * (sigma[0].recover() -
                                                  sigma[0].recover().T.conj())
        ind = get_matrix_index(tp.hsd.S[0].ll_index[1][-1])
        lambda_r_ii[ind.T, ind] = 1.j * (sigma[1].recover() -
                                                  sigma[1].recover().T.conj())
        s_mm = tp.hsd.S[k].recover()
        s_s_i, s_s_ii = np.linalg.eig(s_mm)
        s_s_i = np.abs(s_s_i)
        s_s_sqrt_i = np.sqrt(s_s_i) # sqrt of eigenvalues
        s_s_sqrt_ii = np.dot(s_s_ii * s_s_sqrt_i, dagger(s_s_ii))
        s_s_isqrt_ii = np.dot(s_s_ii / s_s_sqrt_i, dagger(s_s_ii))

        lambdab_r_ii = np.dot(np.dot(s_s_isqrt_ii, lambda_r_ii),s_s_isqrt_ii)
        a_l_ii = np.dot(np.dot(g_s_ii, lambda_l_ii), dagger(g_s_ii))
        ab_l_ii = np.dot(np.dot(s_s_sqrt_ii, a_l_ii), s_s_sqrt_ii)
        lambda_i, u_ii = np.linalg.eig(ab_l_ii)
        ut_ii = np.sqrt(lambda_i / (2.0 * np.pi)) * u_ii
        m_ii = 2 * np.pi * np.dot(np.dot(dagger(ut_ii), lambdab_r_ii),ut_ii)
        T_i,c_in = np.linalg.eig(m_ii)
        T_i = np.abs(T_i)
        channels = np.argsort(-T_i)
        c_in = np.take(c_in, channels, axis=1)
        T_n = np.take(T_i, channels)
        v_in = np.dot(np.dot(s_s_isqrt_ii, ut_ii), c_in)
        return T_n, v_in
    
    def calculate_realspace_dos(self, tp, energy):
        wfs = tp.extended_calc.wfs
        dosg = wfs.gd.zeros(tp.nspins)
        for kpt in wfs.kpt_u:
            s = kpt.s
            q = kpt.q
            sigma = self.calculate_sigma(tp, s, q, energy)

            self.reset_green_function(tp, s, q)
            tp.hsd.G.reset_from_others(tp.hsd.S[q],
                                       tp.hsd.H[s][q],
                                       energy, -1, init=True)
            
            for i in range(tp.hsd.G.lead_num):
                tp.hsd.G.diag_h[i][-1].reset_minus(sigma[i])
            
            gr = np.linalg.inv(tp.hsd.G.recover(True))
            dos_mm = np.dot(gr, tp.hsd.S[q].recover(True))
        
            if wfs.dtype == float:
                dos_mm = np.real(dos_mm).copy()
            wfs.basis_functions.construct_density(dos_mm, dosg[kpt.s], kpt.q)
        wfs.kd.comm.sum(dosg)
        wfs.band_comm.sum(dosg)
        global_dosg = wfs.gd.collect(dosg)
        return global_dosg
       
    def lead_k_matrix(self, tp, l, s, pk, k_vec, hors='S'):
        if hors == 'S':
            h00 = tp.lead_hsd[l].S[pk].recover()
            h01 = tp.lead_couple_hsd[l].S[pk].recover()
        else:
            h00 = tp.lead_hsd[l].H[s][pk].recover()
            h01 = tp.lead_couple_hsd[l].H[s][pk].recover()
        h01 *=  np.exp(2 * np.pi * 1.j * k_vec)
        return h00 + h01 + h01.T.conj()
      
    def lead_scattering_states(self, tp, energy, l, s, q):
        #Calculating the scattering states in electrodes
        #l ---- index of electrode
        #s ---- index of spin
        #q ---- index of local k point
        
        #if it is multi-terminal system, should add a part that can rotate
        # the lead hamiltonian
        MaxLambda = 1e2
        MinErr = 1e-8
        energy += tp.bias[l]
        hes00 = tp.lead_hsd[l].H[s][q].recover() - \
                                    tp.lead_hsd[l].S[q].recover() * energy
        hes01 = tp.lead_couple_hsd[l].H[s][q].recover() - \
                             tp.lead_couple_hsd[l].S[q].recover() * energy
        nb = hes00.shape[-1]
        dtype = hes00.dtype
        A = np.zeros([2*nb, 2*nb], dtype)
        B = np.zeros([2*nb, 2*nb], dtype)
            
        A[:nb, nb:2*nb] = np.eye(nb)
        A[nb:2*nb, :nb] = hes01.T.conj()
        A[nb:2*nb, nb:2*nb] = hes00
            
        B[:nb, :nb] = np.eye(nb)
        B[nb:2*nb, nb:2*nb] = -hes01
            
        from scipy.linalg import eig
        D, V = eig(A, B)
        index = np.argsort(abs(D))
        D = D[index]
        V = V[:, index]
        
        #delete NaN
        index = find(np.abs(D) >= 0)
        D = D[index]
        V = V[:, index]
        
        #delete some unreasonable solutions
        index = find(abs(D) > MaxLambda)
        cutlen = len(D) - index[0]
        index = np.arange(cutlen, len(D) - cutlen)
        D = D[index]
        V = V[:, index]
            
        k = np.log(D) * (-1.j) / 2 / np.pi
        Vk = V[:nb]
            
        #sort scattering states
        proindex = find(abs(k.imag) < MinErr)
            
        if len(proindex) > 0:
            k_sort = np.sort(k[proindex].real)
            index = np.argsort(k[proindex].real)
            k[proindex] = k[proindex[index]].real
            Vk[:, proindex] = Vk[:, proindex[index]]
                
            #normalization the scattering states
            j = proindex[0]
            while j <= proindex[-1]:
                same_k_index = find(abs((k - k[j]).real) < MinErr)
                sk = self.lead_k_matrix(tp, l, s, q, k[j])
                Vk[:, same_k_index] = eig_states_norm(Vk[:, same_k_index],
                                                                         sk)
                j += len(same_k_index)
        return np.array(k[proindex]), np.array(Vk[:, proindex])
                   
    def central_scattering_states(self, tp, energy, s, q):
        #To get the scattering states corresponding to a Bloch vector in lead
        MaxLambda = 1e2
        MinErr = 1e-8
        bc = tp.inner_mol_index
        nc = len(bc)
        molhes = tp.hsd.H[s][q].recover(True) - \
                              tp.hsd.S[q].recover(True) * energy
        blead = []
        lead_hes = []
        lead_couple_hes = []
        for i in range(tp.lead_num):
            blead.append(tp.lead_layer_index[i][-1])
            lead_hes.append(tp.lead_hsd[i].H[s][q].recover() -
                                       tp.lead_hsd[i].S[q].recover() * energy)
            lead_couple_hes.append(tp.lead_couple_hsd[i].H[s][q].recover() -
                                tp.lead_couple_hsd[i].S[q].recover() * energy)

        ex_ll_index = tp.hsd.S[0].ex_ll_index
        
        total_k = []
        total_vk = []
        total_lam = []
        total_v = []
        total_pro_right_index = []
        total_pro_left_index = []
        total_left_index = []
        total_right_index = []
        total_kr = []
        total_kt = []
        total_lambdar = []
        total_lambdat = []
        total_vkr = []
        total_vkt = []
        total_vr = []
        total_vt = []
        total_len_k = []
        total_nblead = []
        total_bA1 = []
        total_bA2 = []
        total_bproin = []
        total_nbB2 = []
        total_bB2 = []
                
        for i in range(tp.lead_num):
            k, vk = self.lead_scattering_states(tp, energy, i, s, q)
            total_k.append(k)
            total_vk.append(vk)
            
            lam = np.exp(2 * np.pi * k * 1.j)
            total_lam.append(lam)
            
            #calculating v = dE/dk
            de2 = 1e-8
            k2, vk2 = self.lead_scattering_states(tp, energy + de2, i, s, q)
            v = de2 / (k2 - k) / 2 / np.pi
            total_v.append(v)
            
            #separating left scaterring states and right scattering states
            #left scattering: lead->mol, right scattering: mol->lead
            proindex = find(abs(k.imag) < MinErr)
            pro_left_index = proindex[find(v[proindex].real < 0)]
            pro_left_index = pro_left_index[-np.arange(len(pro_left_index))
                                                                          - 1]
            
            pro_right_index = proindex[find(v[proindex].real > 0)]
               
            left_index = find(k.imag > MinErr)
            left_index = np.append(left_index, pro_left_index)
            
            right_index = pro_right_index.copy()
            right_index = np.append(right_index, find(k.imag < -MinErr))
            
            total_pro_left_index.append(pro_left_index)
            total_pro_right_index.append(pro_right_index)
            total_left_index.append(left_index)
            total_right_index.append(right_index)
           
            kr = k[left_index]
            kt = k[right_index]
            total_kr.append(kr)
            total_kt.append(kt)
            
            lambdar = np.diag(np.exp(2 * np.pi * kr * 1.j))
            lambdat = np.diag(np.exp(2 * np.pi * kt * 1.j))
            total_lambdar.append(lambdar)
            total_lambdat.append(lambdat)
            
            vkr = np.take(vk, left_index, axis=1)
            vkt = np.take(vk, right_index, axis=1)
            vr = v[pro_left_index]
            vt = v[pro_right_index]
            total_vkr.append(vkr)
            total_vkt.append(vkt)
            total_vr.append(vr)
            total_vt.append(vt)
            
            #abstract basis information
            len_k = len(right_index)
            total_len_k.append(len_k)
            
            #lead i basis seqeunce in whole matrix
            nblead = len(ex_ll_index[i][-1])
            total_nblead.append(nblead)
            bA1 = nc + int(np.sum(total_nblead[:i])) + np.arange(nblead)
            #sequence of the incident wave
            bA2 = nc + int(np.sum(total_len_k[:i])) + np.arange(len_k)
            # the first n in vkt are scattering waves, the rest are the decaying ones
            # the first n in vkr are decaying waves, the rest are the scattering ones
            total_bA1.append(bA1)
            total_bA2.append(bA2)
           
            bproin = np.arange(len(pro_right_index)) + \
                                          len(kr) - len(pro_right_index)
            ### this line need to check it...
            total_bproin.append(bproin)
            nbB2 = len(bproin)
            total_nbB2.append(nbB2)
            bB2 = int(np.sum(total_nbB2[:i])) + np.arange(nbB2)
            total_bB2.append(bB2)

        ind = get_matrix_index(bc)
        Acc = molhes[ind.T, ind]
           
        total_Alc = []
        total_hcl = []
        total_All = []
        total_Acl = []
        total_Bc = []
        total_Bl = []
            
        for i in range(tp.lead_num):
            ind1, ind2 = get_matrix_index(blead[i], bc)
            Alc = molhes[ind1, ind2]
            ind1, ind2 = get_matrix_index(bc, blead[i])
            hcl = molhes[ind1, ind2]
            vkt = total_vkt[i]
            All = np.dot(tp.lead_hsd[i].H[s][q].recover(), vkt) + \
                    np.dot(np.dot(tp.lead_couple_hsd[i].H[s][q].recover(), \
                                                     vkt), total_lambdat[i])
            Acl = np.dot(hcl, vkt)
            bpi = total_bproin[i]
            vkr = total_vkr[i]
            vkr_bpi = np.take(vkr, bpi, axis=1)
            Bc = -np.dot(hcl, vkr_bpi)
            ind = get_matrix_index(bpi)
            Bl = -np.dot(tp.lead_hsd[i].H[s][q].recover(), vkr_bpi) + \
                    np.dot(np.dot(tp.lead_couple_hsd[i].H[s][q].recover(),
                        vkr_bpi) ,total_lambdar[i][ind.T, ind])
            total_Alc.append(Alc)
            total_Acl.append(Acl)
            total_hcl.append(hcl)
            total_All.append(All)
            total_Bc.append(Bc)
            total_Bl.append(Bl)
        total_bc = molhes.shape[-1]
        MatA = np.zeros([total_bc, nc + np.sum(total_len_k)], complex)
        MatB = np.zeros([total_bc, np.sum(total_nbB2)], complex)
        ind = get_matrix_index(np.arange(nc))
        MatA[ind.T, ind] = Acc
        for i in range(tp.lead_num):
            ind1, ind2 = get_matrix_index(total_bA1[i], total_bA2[i])
            MatA[ind1, ind2] = total_All[i]
            ind1, ind2 = get_matrix_index(np.arange(nc), total_bA2[i])
            MatA[ind1, ind2] = total_Acl[i]
            ind1, ind2 = get_matrix_index(total_bA1[i], np.arange(nc))
            MatA[ind1, ind2] = total_Alc[i]
            ind1, ind2 = get_matrix_index(np.arange(nc), total_bB2[i])
            MatB[ind1, ind2] = total_Bc[i]
            ind1, ind2 = get_matrix_index(total_bA1[i], total_bB2[i])
            MatB[ind1, ind2] = total_Bl[i]
        Vx, residues, rank, singular = np.linalg.lstsq(MatA, MatB)
        
        total_vl = []
        for i in range(tp.lead_num):
            total_k[i] = total_k[i][total_pro_right_index[i]]
            total_vl.append(np.take(total_vk[i],
                                    total_pro_right_index[i], axis=1))
         
        total_vc = []
        t = []
        for i in range(tp.lead_num):
            t.append([])
            for j in range(tp.lead_num):
                t[i].append([])
        for i in range(tp.lead_num):
            ind1, ind2 = get_matrix_index(np.arange(nc), total_bB2[i])
            total_vc.append(Vx[ind1, ind2])
            for j in range(tp.lead_num):
                bx = total_bA2[j]
                bx = bx[:len(total_pro_right_index[j])]
                ind1, ind2 = get_matrix_index(bx, total_bB2[i])
                #t[j].append(Vx[ind1, ind2])
                t[j][i] = Vx[ind1, ind2]
                for m in range(len(total_pro_left_index[i])):
                    for n in range(len(total_pro_right_index[j])):
                        t[j][i][n, m] *= np.sqrt(abs(total_vt[i][n]/
                                                          total_vr[j][m]))
        return np.array(t), np.array(total_vc), \
                                         np.array(total_k), np.array(total_vl)
            
    def cal_sstates(self, tp, energies, s, q):
        #To see how much the transmision from one orbital(local basis) in lead a
        # to the orbital in the lead b, the transmission is decomposed to
        # Bloch wave in leads
        MinErr = 1e-8
        MaxLambda = 1 + MinErr
        
        energies = np.real(energies)
        ne = len(energies)
        t_all = np.zeros([ne, tp.lead_num, tp.lead_num])
        t_lead_all = []
        for i in range(tp.lead_num):
            t_lead_all.append([])
            for j in range(tp.lead_num):
                t_lead_all[i].append(np.zeros([tp.nblead[i],
                                                     tp.nblead[j], ne]))
        total_t = []
        total_vc = []
        total_k = []
        total_vl = []
        sk1 = []
        sk2 = []
        for n, energy in enumerate(energies):
            t, vc, k, vl = self.central_scattering_states(tp, energy, s, q)
            total_t.append(t)
            total_vc.append(vc)
            total_k.append(k)
            total_vl.append(vl)
            for i in range(tp.lead_num):
                for j in range(tp.lead_num):
                    t0 = t[i][j]
                    t_all[n, i, j] = np.sum(np.dot(t0.T.conj(), t0))
                    for m1 in range(t0.shape[0]):
                        sk1.append(self.lead_k_matrix(tp, i, s, q, k[i][m1], 'S'))
                    for m2 in range(t0.shape[1]):
                        sk2.append(self.lead_k_matrix(tp, i, s, q, k[j][m2], 'S'))
                    for m1 in range(t0.shape[0]):
                        for m2 in range(t0.shape[1]):
                            w1 = (vl[j][:, m1].conj() * np.dot(sk1[m1],
                                                        vl[j][:, m1])).real
                            w1[find(w1 < 0)] = 0
                            w1 /= np.sum(w1)
                            
                            w2 = (vl[i][:, m2].conj() * np.dot(sk2[m2],
                                                        vl[i][:, m2])).real
                            w2[find(w2 < 0)] = 0
                            w2 /= np.sum(w2)
                            
                            t_lead_all[i][j][:, :, n] += abs(t0[m1,
                                          m2]) ** 2 * np.dot(w2, w1.T.conj())
        return np.array(t_lead_all), np.array(t_all)

    
    def save_ele_step(self, tp):
        if not self.matrix_foot_print:
            fd = open('matrix_sample', 'wb')
            sample = Tp_Sparse_Matrix(complex, tp.hsd.ll_index)
            sample.reset_from_others(tp.hsd.S[0], tp.hsd.H[0][0], 1.,
                                                      -1.j, init=True)
            pickle.dump(sample, fd, 2)
            fd.close()
            self.matrix_foot_print = True
        #selfconsistent calculation: extended_calc ---> extended_atoms
        #non_selfconsistent calculation: extended_calc ---> original_atoms
        
        calc = tp.extended_calc
        gd = calc.gd
        finegd = calc.hamiltonian.finegd

        if not tp.use_qzk_boundary:
            nt_sG = tp.density.nt_sG
            nt_sg = tp.density.nt_sg
        else:
            nt_sG = calc.density.nt_sG
            nt_sg = calc.density.nt_sg
        vt_sG = calc.hamiltonian.vt_sG
        vt_sg = calc.hamiltonian.vt_sg
        data = self.data
        flag = 'ele_' + str(self.n_ele_step) + '_'
        nt = []
        vt = []
        ntg = []
        vtg = []
        for s in range(tp.nspins):
            nts = aa1d(nt_sG[s])
            vts = aa1d(vt_sG[s]) * Hartree
            ntgs = aa1d(nt_sg[s])
            vtgs = aa1d(vt_sg[s]) * Hartree
            nt.append(nts)
            vt.append(vts)
            ntg.append(ntgs)
            vtg.append(vtgs)

        data[flag + 'nt'] = np.array(nt)
        data[flag + 'vt'] = np.array(vt)
        data[flag + 'ntg'] = np.array(ntg)
        data[flag + 'vtg'] = np.array(vtg)
        data[flag + 'vbarg'] = aa1d(calc.hamiltonian.vbar_g)
        data[flag + 'df'] = np.diag(tp.hsd.H[0][0].recover(True))
        data[flag + 'dd'] = np.diag(tp.hsd.D[0][0].recover(True))
        
        if not tp.use_qzk_boundary:
            gd = tp.finegd
            rhot_g = tp.density.rhot_g
            rho = aa1d(rhot_g)
            data[flag + 'rho'] = np.array(rho)
        else:
            gd = calc.finegd
            rhot_g = calc.density.rhot_g
            rho = aa1d(rhot_g)
            data[flag + 'rho'] = np.array(rho)
       
        gd = finegd
        vHt_g = calc.hamiltonian.vHt_g
        vHt = aa1d(vHt_g) * Hartree
        data[flag + 'vHt'] = vHt
        self.n_ele_step += 1
      
    def save_bias_step(self, tp):
        tp.log('-> save_bias_step()')
        tp.text('----- saving bias step {0} ------'.format(self.n_bias_step))
        if not self.overhead_data_saved:
            tp.log('   save_overhead_data()')
            self.save_overhead_data(tp)
            self.overhead_data_saved = True
        ks_map = tp.my_ks_map

        if 'tc' in tp.analysis_data_list:
            tp.log('   collect_transmission_and_dos()')
            tc, dos = self.collect_transmission_and_dos(tp)
            if not tp.non_sc:
                tp.log('   calculate_current()')
                current = self.calculate_current(tp, tc)
            else:
                current = np.array(0)
            self.data['tc'] = tc
            self.data['dos'] = dos
            self.data['current'] = current
        tp.log('   abstract_d_and_v()')
        nt, vt, ntx, vtx, nty, vty = self.abstract_d_and_v(tp)
        for name in ['nt', 'vt', 'ntx', 'vtx', 'nty', 'vty']:
                self.data[name] = eval(name)
        if tp.non_sc or tp.analysis_mode:
            force = None
            contour = None
        else:
            if 'force' in tp.analysis_data_list:
                if not tp.use_qzk_boundary:
                    force = tp.calculate_force() * Hartree / Bohr
                else:
                    force = tp.extended_calc.get_forces(tp.extended_atoms
                                                    )[:len(tp.atoms)]
            else:
                force = None
            tp.F_av = None
            tp.log('   collect_contour()')
            contour = self.collect_contour(tp)
    
        tp.log('   collect_charge()')
        charge = self.collect_charge(tp)
        magmom = tp.occupations.magmom
        local_magmom = tp.get_magnetic_moments()

        lead_fermi = np.array(tp.lead_fermi)
        lead_pairs = np.array(self.lead_pairs)
        bias = np.array(tp.bias)
        gate = np.array(tp.gate)
        magmom = np.array(magmom)
        local_magmom = np.array(local_magmom)
        for name in ['lead_fermi', 'lead_pairs', 'bias', 'gate',
                                      'charge', 'magmom', 'local_magmom']:
            self.data[name] = eval(name)
        # do not include contour now because it is a dict, not a array able to
        # collect, but will do it at last
        #self.data = gather_ndarray_dict(self.data, tp.contour.comm)
        self.data['contour'] = contour
        self.data['force'] = force
        self.data['kpt_rank'] = tp.wfs.kd.comm.rank
        self.data['kpt_size'] = tp.wfs.kd.comm.size
        self.data['domain_rank'] = tp.gd.comm.rank
        self.data['domain_parpos'] = tp.gd.parpos_c
        self.data['domain_parsize'] = tp.gd.parsize_c
        ns, npk, nlp, ne = tp.nspins, tp.npk, len(self.lead_pairs), len(self.energies)
        self.data['transmission_dimension'] = (ns, npk, nlp, ne)
        nb = tp.nbmol_inner
        self.data['dos_dimension'] = (ns, npk, nb, ne)

        if tp.non_sc:
            self.data['total_energy'] = tp.guess_total_energy
        for condition, obj, name in tp.special_datas:
            if eval(condition):
                self.data[name] = eval(obj)
       
        tp.log('   tio.save_data()')
        tp.tio.save_data(self, option='Analysis', bias_step=self.n_bias_step)
        tp.log('   tio.arrange_analysis_data()')
        tp.tio.arrange_analysis_data(self.n_bias_step,
                                            self.n_ion_step, tp.analysis_mode)
        #if world.rank == 0:
        #    if tp.analysis_mode:
        #        filename = '/abias_step_' + str(self.n_bias_step)
        #    else:
        #        filename = '/bias_step_' + str(self.n_bias_step)
        #    fd = file('analysis_data/ionic_step_' + str(self.n_ion_step)
        #              + filename, 'wb')
        #    cPickle.dump(self.data, fd, 2)
        #    fd.close()
        self.data = {}
        self.n_ele_step = 0
        self.n_bias_step += 1
        tp.log('<- save_bias_step()')
        

    def collect_transmission_and_dos(self, tp, energies=None, nids=None):
        tp.log('-> collect_transmission_and_dos()')
        if energies is None:
            energies = self.my_energies
        if nids is None:
            nids = self.my_nids
      
        nlp = len(self.lead_pairs)
        ne = len(energies)
        nbmol = tp.nbmol_inner
        ns, npk = tp.my_nspins, tp.my_npk
        local_tc_array = np.empty([ns, npk, nlp, ne], float)
        local_dos_array = np.empty([ns, npk, nbmol, ne], float)
        
        tp.log('  nr. energies: {0}'.format(len(energies)))
        tp.log('  nids: {0}'.format(nids))
        for s in range(ns):
            for q in range(npk):
                for e, energy, nid in zip(range(len(energies)), energies, nids):
                    tp.log('  calculate_dos() {0} {1} {2}'.format(s,q,e))
                    local_dos_array[s, q, :, e] = self.calculate_dos(tp, s, q,
                                                               energy, nid)
                    tp.log('  calculate_transmission() {0} {1} {2}'.format(s,q,e))
                    local_tc_array[s, q, :, e] =  self.calculate_transmission(tp, s,
                                                            q, energy, nid)

        #kpt_comm = tp.wfs.kd.comm
        #ns, npk = tp.nspins, tp.npk
        #if kpt_comm.rank == 0:
        #    tc_array = np.empty([ns, npk, nlp, ne], float)
        #    dos_array = np.empty([ns, npk, nbmol, ne], float)
        #else:
        #    tc_array = None
        #    dos_array = None
        #kpt_comm.gather(local_tc_array, 0, tc_array)
        #kpt_comm.gather(local_dos_array, 0, dos_array)
        tp.log('<- collect_transmission_and_dos()')
        return local_tc_array, local_dos_array
    
    def collect_charge(self, tp):
        nbmol = tp.nbmol + np.sum(tp.nblead)
        ns, npk = tp.my_nspins, tp.my_npk
        local_charge_array = np.zeros([ns, npk, nbmol])
        for s in range(ns):
            for q in range(npk):
                    local_charge_array[s, q] = \
                                     self.calculate_charge_distribution(tp, s, q)
        charge_array = np.sum(local_charge_array, axis=1)
        return charge_array

    def collect_contour(self, tp):
        my_eq_contour = {}
        my_ne_contour = {}
        my_loc_contour = {}
        num = 0
        for s in range(tp.my_nspins):
            for q in range(tp.my_npk):
                flag = str(tp.my_ks_map[num, 0]) + str(tp.my_ks_map[num, 1])
                my_eq_contour[flag] = np.array(tp.eqpathinfo[s][q].energy)
                my_ne_contour[flag] = np.array(tp.nepathinfo[s][q].energy)
                if not tp.ground:
                    my_loc_contour[flag] = np.array(tp.locpathinfo[s][q].energy)
        #eq_contour = gather_ndarray_dict(my_eq_contour, tp.wfs.kd.comm)
        #ne_contour = gather_ndarray_dict(my_ne_contour, tp.wfs.kd.comm)
        #if not tp.ground:
        #    loc_contour = gather_ndarray_dict(my_loc_contour, tp.wfs.kd.comm)
        #else:
        #    loc_contour = None
        contour = {'eq': my_eq_contour, 'ne': my_ne_contour, 'loc': my_loc_contour}
        return contour
 
    def save_ion_step(self, tp):
        if world.rank == 0:
            fd = open('analysis_data/ionic_step_' +
                  str(self.n_ion_step) +'/positions', 'wb')
            pickle.dump(tp.atoms.positions, fd, 2)
            fd.close()
        self.n_bias_step = 0
        self.n_ion_step += 1
        if world.rank == 0:
            dirname = 'analysis_data/ionic_step_' + str(self.n_ion_step)
            if not os.access(dirname, os.F_OK):
                os.mkdir(dirname)
        world.barrier()
 
    def abstract_d_and_v(self, tp):
        calc = tp.extended_calc
        gd = calc.gd
        if not tp.use_qzk_boundary:
            nt_sG = tp.density.nt_sG
        else:
            nt_sG = calc.density.nt_sG
        vt_sG = calc.hamiltonian.vt_sG
        nt = []
        vt = []
        ntx = []
        nty = []
        vtx = []
        vty = []
        for s in range(tp.nspins):
            nts = aa1d(nt_sG[s])
            vts = aa1d(vt_sG[s])
            ntsx = aa2d(nt_sG[s], 0)
            vtsx = aa2d(vt_sG[s], 0)
            ntsy = aa2d(nt_sG[s], 1)
            vtsy = aa2d(vt_sG[s], 1)
            nt.append(nts)
            vt.append(vts)
            ntx.append(ntsx)
            vtx.append(vtsx)
            nty.append(ntsy)
            vty.append(vtsy)
        nt = np.array(nt)
        vt = np.array(vt)
        ntx = np.array(ntx)
        vtx = np.array(vtx)
        nty = np.array(nty)
        vty = np.array(vty)
        return nt, vt, ntx, vtx, nty, vty
    
    def calculate_current(self, tp, tc_array, lead_pair_index=0):
        current = np.array([0, 0], complex)
        contour = tp.contour
        kt = 0.02
        fd = fermidistribution
        lead_ef1 = contour.leadfermi[self.lead_pairs[lead_pair_index][0]]
        lead_ef2 = contour.leadfermi[self.lead_pairs[lead_pair_index][1]]
        if lead_ef2 > lead_ef1:
            lead_ef1, lead_ef2 = lead_ef2, lead_ef1
        interval = np.real(self.my_energies[1] - self.my_energies[0])
        tc_all = np.sum(tc_array, axis=1) / tp.npk
        fermi_factor = fd(self.my_energies - lead_ef1, kt) - fd(
                                         self.my_energies - lead_ef2, kt)
        for s in range(tp.my_nspins):
            current[s] = np.sum(tc_all[s, lead_pair_index] * fermi_factor *
                                                          self.my_weights)
        current = np.array(current)
        #tp.contour.comm.sum(current)
        return current
         
class Transport_Plotter:
    def __init__(self, mode=0):
        self.my_options = False
        self.ion_step = 0
        self.bias_step = 0
        self.ele_step = 0
        self.plot_setup()
        self.initialize()
        self.mode = mode
    
    def initialize(self):
        self.xlabels = {}
        self.ylabels = {}
        names = ['tc', 'dos', 'nt', 'vt']
        xls = ['Energy(eV)', 'Energy(eV)', 'Transport Axis', 'Transport Axis']
        yls = ['Transmission Coefficient', 'Density of States(Electron/eV)',
               'Electron Density', 'Effective Potential']
        for name, xl, yl in zip(names, xls, yls):
            self.xlabels[name] = xl
            self.ylabels[name] = yl

    def read_overhead(self):
        fd = open('analysis_overhead', 'rb')
        atoms, basis_information, contour_information = pickle.load(fd)
        fd.close()
        self.atoms = atoms
        self.basis = basis_information
        self.contour = contour_information
 
    def plot_setup(self):
        from matplotlib import rcParams
        rcParams['xtick.labelsize'] = 18
        rcParams['ytick.labelsize'] = 18
        rcParams['legend.fontsize'] = 18
        rcParams['axes.titlesize'] = 18
        rcParams['axes.labelsize'] = 18
        rcParams['font.size'] = 18
        rcParams['figure.subplot.left'] = 0.15

    def get_data(self, bias_step, ion_step):
        if self.mode == 0:
            fd = open('analysis_data/ionic_step_' + str(ion_step) +
                  '/bias_step_' + str(bias_step), 'rb')
        else:
            fd = open('analysis_data/ionic_step_' + str(ion_step) +
                  '/abias_step_' + str(bias_step), 'rb')
        data = pickle.load(fd)
        fd.close()
        return data
    
    def get_info(self, name, bias_step, ion_step=0):
        data = self.get_data(bias_step, ion_step)
        if 'newform' in data and data['newform'] == True:
            info = self.collect_info(data, name)
        else:
            if name in ['tc', 'dos']:
                info = data['ER_0_' + name]
                data_name = 'ER_1_' + name
                n = 1
                while data_name in data:
                    info = np.append(info, data[data_name], axis=-1)
                    n += 1
                    data_name = 'ER_' + str(n) + '_' + name
            else:
                info = data[name]
        return info

    def collect_info(self, data, name):
        kpt_size = data['kpt_size']
        domain_parsize = data['domain_parsize']
        domain_size = np.product(domain_parsize)
        if name == 'vt' or name == 'nt':
            vt = []
            for i in range(domain_parsize[2]):
                vt.append([])
            for i in range(domain_size):
                d1, d2, d3 =  (i // (domain_parsize[1] * domain_parsize[2]),
                               i % (domain_parsize[1] *
                               domain_parsize[2]) // domain_parsize[2],
                               i % domain_parsize[2])
                lvt = data['K_0D_' + str(i) + '_' + name]
                #lvt = np.sum(lvt, axis=1) / lvt.shape[1]
                #lvt = np.sum(lvt, axis=1) / lvt.shape[1]
                vt[d3].append(lvt)
            for i in range(domain_parsize[2]):
                vt[i] = np.sum(vt[i],axis=0) / len(vt[i])
            output = np.concatenate(vt, axis=-1)
        elif name =='vty' or name =='nty':
            vty = []
            for i in range(domain_parsize[0]):
                vty.append([])
                for j in range(domain_parsize[2]):
                    vty[i].append([])
            for i in range(domain_size):
                d1, d2, d3 =  (i // (domain_parsize[1] * domain_parsize[2]),
                               i % (domain_parsize[1] *
                               domain_parsize[2]) // domain_parsize[2],
                               i % domain_parsize[2])
                lvty = data['K_0D_' + str(i) + '_' + name]
                vty[d1][d3].append(lvty)
            for i in range(domain_parsize[0]):
                for j in range(domain_parsize[2]):
                    vty[i][j] = np.sum(vty[i][j],axis=0) / len(vty[i][j])
            output = np.concatenate(vty, axis=-1)
            output = np.concatenate(output, axis=-1)
        elif name =='vtx' or name =='ntx':
            vtx = []
            for i in range(domain_parsize[1]):
                vtx.append([])
                for j in range(domain_parsize[2]):
                    vtx[i].append([])
            for i in range(domain_size):
                d1, d2, d3 =  (i // (domain_parsize[1] * domain_parsize[2]),
                               i % (domain_parsize[1] *
                               domain_parsize[2]) // domain_parsize[2],
                               i % domain_parsize[2])
                lvtx = data['K_0D_' + str(i) + '_' + name]
                vtx[d2][d3].append(lvtx)
            for i in range(domain_parsize[1]):
                for j in range(domain_parsize[2]):
                    vtx[i][j] = np.sum(vtx[i][j],axis=0) / len(vtx[i][j])
            output = np.concatenate(vtx, axis=-1)
            output = np.concatenate(output, axis=-1)
        elif name == 'vHt' or name == 'rho':
            vHt = []
            for i in range(domain_parsize[2]):
                vHt.append([])
            for i in range(domain_size):
                d1, d2, d3 =  (i // (domain_parsize[1] * domain_parsize[2]),
                               i % (domain_parsize[1] *
                               domain_parsize[2]) // domain_parsize[2],
                               i % domain_parsize[2])
                lvHt = data['K_0D_' + str(i) + '_' + name]
                lvHt = np.sum(lvHt, axis=0) / lvHt.shape[0]
                lvHt = np.sum(lvHt, axis=0) / lvHt.shape[0]
                vHt[d3].append(lvHt)
            for i in range(domain_parsize[2]):
                vHt[i] = np.sum(vHt[i],axis=0) / vHt[i].shape[0]
            output = np.concatenate(vHt, axis=-1)
        elif name == 'tc':
            ns, npk, npl, ne = data['transmission_dimension']
            tc = []
            for i in range(kpt_size):
                ltc = []
                for j in range(domain_size):
                    ltc.append(data['K_' + str(i) + 'D_' + str(j) + '_tc'])
                ltc = np.concatenate(ltc, axis=-1)
                tc.append(ltc)
            output = np.array(tc)
            output.shape = (ns, npk, npl, ne)
        elif name == 'dos':
            ns, npk, nb, ne = data['dos_dimension']
            dos = []
            for i in range(kpt_size):
                ldos = []
                for j in range(domain_size):
                    ldos.append(data['K_' + str(i) + 'D_' + str(j) + '_dos'])
                ldos = np.concatenate(ldos, axis=-1)
                dos.append(ldos)
            output = np.array(dos)
            output.shape = (ns, npk, nb, ne)
        elif name == 'charge':
            data = pickle.load(open('temperary_data/KC_0_DC_0AD_bias_0', 'rb'))
            data1 = pickle.load(open('temperary_data/KC_1_DC_0AD_bias_0', 'rb'))
            output = [data['charge'], data1['charge']]
            output=np.sum(output,axis=1)
        elif name == 'current':
            # temperary lines
            # ! THa: Hack, load energy scale from saved file
            try:
                ee  = np.load('plot_energy_range.npy')
                enp = len(ee)
            except IOError:
                print('WARNING: could not find file plot_energy_range.npy,\
                        fall back to default: np.linspoace(-5,5,201)')
                ee = np.linspace(-5,5,201)
            enp = len(ee)
            interval = np.abs(ee[0] - ee[-1]) / enp
            # ! THa: Hack end
            kt = 0.02
            lead_ef1 = data['bias'][0]
            lead_ef2 = data['bias'][1]
            ns, npk, npl, ne = data['transmission_dimension']
            tc = []
            for i in range(kpt_size):
                ltc = []
                for j in range(domain_size):
                    ltc.append(data['K_' + str(i) + 'D_' + str(j) + '_tc'])
                ltc = np.concatenate(ltc, axis=-1)
                tc.append(ltc)
            tc = np.array(tc)
            tc.shape = (ns, npk,  ne)
            tc = np.sum(tc, axis=1) / npk
            fd = fermidistribution
            fermi_factor = fd(ee - lead_ef1, kt) - fd(ee - lead_ef2, kt)
            output = np.sum(tc * fermi_factor * interval, axis=-1)
        else:
            output = data[name]
        return output

    def get_positions(self, ion_step=0):
        fd = open('analysis_data/ionic_step_' + str(ion_step) + '/positions',
                  'rb')
        positions = pickle.load(fd)
        fd.close()
        return positions

    def process(self, info, s=0, k=None, lp=None):
        if s is None:
            info = np.sum(info, axis=0) / info.shape[0]
        else:
            info = info[s]
        if k is None:
            info = np.sum(info, axis=0) / info.shape[0]
        else:
            info = info[k]
        if lp is not None:
            info = info[lp]
        return info
        
    def tc(self, bias_step, ion_step=0, lp=0, s=0, k=None):
        info = self.get_info('tc', bias_step, ion_step)
        return self.process(info, s, k, lp)
    
    def dos_array(self, bias_step, ion_step=0, s=0, k=None):
        info = self.get_info('dos', bias_step, ion_step)
        return self.process(info, s, k)
    
    def dos(self, bias_step, ion_step=0, s=0, k=None):
        info = self.get_info('dos', bias_step, ion_step)
        info = self.process(info, s, k)
        return np.sum(info, axis=-2)
 
    def partial_dos(self, bias_step, ion_step=0, s=0, k=None,
                    atom_indices=None, orbital_type=None, direction=None):
        self.read_overhead()
        dos_array = self.dos_array(bias_step, ion_step, s, k)
        orbital_indices = self.basis['orbital_indices']
        orbital_map = {'S': 0, 'P': 1, 'D': 2, 'F': 3}
        direction_map = {'x': 2, 'y': 0, 'z': 1, 'xy': 0, 'yz': 1,
                         'z2r2': 2, 'xz': 3, 'x2y2': 4}
        if direction is None:
            direction_index = np.zeros([orbital_indices.shape[0]]) + 1
        else:
            directions = []
            num = -1
            oio = 0
            for oi in orbital_indices:
                if oi[1] != oio:
                    num = -1
                oio = oi[1]
                if oi[1] != orbital_map[orbital_type]:
                    directions.append(0)
                else:
                    num += 1
                    if num == direction_map[direction]:
                        directions.append(1)
                    else:
                        directions.append(0)
            direction_index = np.array(directions)
            
        if orbital_type is None:
            orbital_index = np.zeros([orbital_indices.shape[0]]) + 1
        else:
            orbital_index = orbital_indices[:, 1] - orbital_map[
                                                      orbital_type] ==  0
        atom_index = np.zeros([orbital_indices.shape[0]]) + 1
        if atom_indices is not None:
            atom_index -= 1
            for i in atom_indices:
                atom_index += orbital_indices[:, 0] - i == 0
        pdos = []
        for i in range(dos_array.shape[1]):
            pdos.append(np.sum(dos_array[:,i] * orbital_index * atom_index * direction_index))
        pdos = np.array(pdos)
        return pdos

    def charge(self, bias_step, ion_step=0, s=0, atom_indices=None,
               orbital_type=None, direction=None):
        self.read_overhead()
        charge_array = self.get_info('charge', bias_step, ion_step)
        orbital_indices = self.basis['orbital_indices']
        orbital_map = {'S': 0, 'P': 1, 'D': 2, 'F': 3}
        direction_map = {'x': 2, 'y': 0, 'z': 1, 'xy': 0, 'yz': 1,
                         'z2r2': 2, 'xz': 3, 'x2y2': 4}
        if direction is None:
            direction_index = np.zeros([orbital_indices.shape[0]]) + 1
        else:
            directions = []
            num = -1
            oio = 0
            for oi in orbital_indices:
                if oi[1] != oio:
                    num = -1
                oio = oi[1]
                if oi[1] != orbital_map[orbital_type]:
                    directions.append(0)
                else:
                    num += 1
                    if num == direction_map[direction]:
                        directions.append(1)
                    else:
                        directions.append(0)
            direction_index = np.array(directions)

        if orbital_type is None:
            orbital_index = np.zeros([orbital_indices.shape[0]]) + 1
        else:
            orbital_index = orbital_indices[:, 1] - orbital_map[
                                                      orbital_type] ==  0
        n_atom_basis = len(orbital_index)
        charge_array = charge_array[s, :n_atom_basis]
        atom_index = np.zeros([orbital_indices.shape[0]]) + 1
        if atom_indices is not None:
            atom_index -= 1
            for i in atom_indices:
                atom_index += orbital_indices[:, 0] - i == 0
        charge = np.sum(charge_array * atom_index * orbital_index * direction_index)
        return charge
       
    def iv(self, nsteps=16, spinpol=False):
        current = []
        bias = []
        for i in range(nsteps):
            bias_list = self.get_info('bias', i)
            bias.append(bias_list[0]-bias_list[1])
            current.append(self.get_info('current', i))
        unit = 6.624 * 1e3
        current = np.array(current) * unit / (Hartree * 2 * np.pi)
        if not spinpol:
            current *= 2
        bias = np.array(bias)
        return bias, current

    def diamond(self, bias_step, ionic_step=0, dense_level=1, spinpol=False):
        tc = self.tc(bias_step, ionic_step)
        if dense_level > 1:
            from scipy import interpolate
            ee = np.linspace(-5, 5, 201)
            new_ee = np.linspace(-5, 5, dense_level * 201)
            tck = interpolate.splrep(ee, tc, s=0)
            new_tc = interpolate.splev(new_ee, tck, der=0)
            tc = new_tc
        de = 0.05 / dense_level
        nb = 30 * dense_level
        ng = 140 * dense_level
        currents = np.zeros([nb, ng])
        for i in range(ng):
            for j in range(nb):
                begin = i + nb - j
                end = i + nb + j
                currents[j, i] = np.sum(tc[begin:end])
        unit = 6.624 * 1e3
        currents = np.array(currents) * de * unit / (Hartree * 2 * np.pi)
        if not spinpol:
            currents *= 2
        return currents
           
    def tvs(self, nsteps=16, spinpol=True):
        bias, current = self.iv(nsteps,  spinpol)
        current *= 1e-6 # switch unit to Ampier
        ydata = np.log(abs(current) / (bias ** 2))
        xdata = 1 / bias
        return xdata, ydata






# -*- coding: utf-8 -*-
import numpy as np
from math import sqrt
from ase.units import Hartree
from ase import Atoms
from gpaw import GPAW, FermiDirac, PoissonSolver
from gpaw.test import equal

##############################################################################
## Define a function that returns the band gab
## defined as the difference between the bottom of the conduction band and the
## top of the valence band.
def band_gab(calc):
    ef = calc.get_fermi_level()
    Nb = calc.wfs.bd.nbands
    w_k = calc.wfs.kd.weight_k
    x = 0
    nspin=calc.get_number_of_spins()
    energies = np.empty(len(w_k) * Nb*nspin)
    for spin in np.arange(nspin):
        for k, w in enumerate(w_k):
            energies[x:x + Nb] = calc.get_eigenvalues(k,spin)
            x += Nb
    index1=np.where(energies-ef<=0)
    index2=np.where(energies-ef>0)

    Vb=max(energies[index1[0]])-ef
    Cb=min(energies[index2[0]])-ef
    return Cb-Vb





##############################################################################
## Setup up bulk NiO in an antiferromagnetic configuration
name='Hubbard_test_on_NiO'
a = 4.19 # Lattice constants
b=a/sqrt(2)
m=2
k=2 # Number of k-points
atoms = Atoms(symbols='Ni2O2',
              pbc=True,
              cell=(b, b, a),
              positions=[(0, 0, 0),
                         (b/2, b/2, a/2),
                         (0, 0, a/2),
                         (b/2, b/2, 0)],
              magmoms=(m,-m,0,0)
              )

##############################################################################
## Setup the calculator
calc = GPAW(
    h=0.25,
    occupations=FermiDirac(width=0.05),
    poissonsolver=PoissonSolver(nn='M', relax='J'),
    setups={'Ni': '10'},
    convergence={'eigenstates':8e-4,'density': 1.0e-2,'energy': 0.1},
    #txt=name+'.txt',
    kpts=(k, k, k),
    xc='PBE')

atoms.set_pbc(1)
atoms.set_calculator(calc)

##############################################################################
## Find the  ground-state and get the band gab
e1 = atoms.get_potential_energy()
Eg_non_Hub = band_gab(calc)

# Setup 6eV Hubbard U on the d-orbitals (l=2) of Ni atoms:
calc.set(setups={'Ni': '10:d,6.0'})
## Make ready for scf with the DFT+U functional and converge this new system
## and get new band bag.....which should be much larger:
e2 = calc.get_potential_energy()
Eg_Hub = band_gab(calc)

equal(Eg_Hub, 4.7, 0.2)
equal(Eg_non_Hub, 0.8, 0.1)



import sys

from ase.calculators.jacapo.pw91_psp import defaultpseudopotentials as psp

from ase.test.tasks.dcdft import DeltaCodesDFTTask

from ase.tasks.calcfactory import calculator_factory

if len(sys.argv) == 1:
    xc = 'PBE'
    run = None
elif len(sys.argv) == 2:
    xc = 'PBE'
    run = sys.argv[1]
elif len(sys.argv) == 3:
    xc = sys.argv[1]
    run = sys.argv[2]

if run is not None:
    tag = 'scf_dcdft_%s_pw_%s' % (xc.lower(), run)
else:
    tag = 'scf_dcdft_%s_pw_jacapo' % (xc.lower())

if run is not None:
    tag += '_%s' % run

class Task(DeltaCodesDFTTask):
    def __init__(self, **kwargs):
        DeltaCodesDFTTask.__init__(self, **kwargs)

    def calculate(self, name, atoms):
        data = DeltaCodesDFTTask.calculate(self, name, atoms)
        try:
            steps = atoms.get_calculator().get_number_of_iterations()
        except (AttributeError, NotImplemented):
            steps = None
        data['calculator steps'] = steps
        return data

calcfactory = calculator_factory('jacapo',
                                 xc=xc,
                                 pw=340,
                                 dw=340,
                                 symmetry=True,
                                 ft=0.1,
                                 # need to be set explicitly
                                 spinpol=True,
                                 )

task = Task(
    calcfactory=calcfactory,
    tag=tag,
    use_lock_files=True,
    )

if __name__ == '__main__':
    keys = set(psp.keys()).intersection(set(task.collection.names))
    for m in ['Zn', 'Zr']:
        keys.remove(m)  # do not converge
    task.run(keys)



from math import cos, sin

from ase import Atoms
from ase.calculators.tip3p import (TIP3P, epsilon0, sigma0, rOH, thetaHOH,
                                   set_tip3p_charges)
from ase.calculators.qmmm import EIQMMM, LJInteractions, Embedding
from ase.constraints import FixInternals
from ase.optimize import BFGS
from gpaw import GPAW

r = rOH
a = thetaHOH

interaction = LJInteractions({('O', 'O'): (epsilon0, sigma0)})

for selection in [[0, 1, 2], [3, 4, 5]]:
    name = ''.join(str(i) for i in selection)
    dimer = Atoms('H2OH2O',
                  [(-r * cos(a / 2), r * sin(a / 2), 0),
                   (-r * cos(a / 2), -r * sin(a / 2), 0),
                   (0, 0, 0),
                   (-r * cos(a), 0, r * sin(a)),
                   (-r, 0, 0),
                   (0, 0, 0),
                   ])
    dimer.positions[3:, 0] += 2.8
    dimer.constraints = FixInternals(
        bonds=[(r, (0, 2)), (r, (1, 2)),
               (r, (3, 5)), (r, (4, 5))],
        angles=[(a, (0, 2, 1)), (a, (3, 5, 4))])

    set_tip3p_charges(dimer)
    dimer.calc = EIQMMM(selection,
                        GPAW(txt=name + '.txt', h=0.17),
                        TIP3P(),
                        interaction,
                        vacuum=4,
                        embedding=Embedding(rc=0.2),
                        output=name + '.out')
    print(dimer.get_potential_energy())
    opt = BFGS(dimer, trajectory=name + '.traj')
    opt.run(0.02)
    
    c = dimer.constraints
    dimer.constraints = None
    monomer = dimer[selection]
    dimer.constraints = c
    monomer.center(vacuum=4)
    monomer.calc = GPAW(txt=name + 'M.txt', h=0.17)
    e0 = monomer.get_potential_energy()
    print(dimer.get_potential_energy() - e0, dimer.get_distance(2, 5))
    opt = BFGS(monomer, trajectory=name + 'M.traj')
    opt.run(0.02)



"""Wigner-Seitz truncated coulomb interaction.

See:

    Ravishankar Sundararaman and T. A. Arias:
    Phys. Rev. B 87, 165122 (2013)
    
    Regularization of the Coulomb singularity in exact exchange by
    Wigner-Seitz truncated interactions: Towards chemical accuracy
    in nontrivial systems
"""
from __future__ import print_function
import sys
from math import pi

import numpy as np
from ase.units import Bohr

import gpaw.mpi as mpi
from gpaw.utilities import erf
from gpaw.fftw import get_efficient_fft_size
from gpaw.grid_descriptor import GridDescriptor


class WignerSeitzTruncatedCoulomb:
    def __init__(self, cell_cv, nk_c, txt=sys.stdout):
        self.nk_c = nk_c
        bigcell_cv = cell_cv * nk_c[:, np.newaxis]
        L_c = (np.linalg.inv(bigcell_cv)**2).sum(0)**-0.5
        
        rc = 0.5 * L_c.min()
        print('Inner radius for %dx%dx%d Wigner-Seitz cell: %.3f Ang' %
              (tuple(nk_c) + (rc * Bohr,)), file=txt)
        
        self.a = 5 / rc
        print('Range-separation parameter: %.3f Ang^-1' % (self.a / Bohr),
              file=txt)
        
#        nr_c = [get_efficient_fft_size(2 * int(L * self.a * 1.5))
        nr_c = [get_efficient_fft_size(2 * int(L * self.a * 3.0))
                for L in L_c]
        print('FFT size for calculating truncated Coulomb: %dx%dx%d' %
              tuple(nr_c), file=txt)
        
        self.gd = GridDescriptor(nr_c, bigcell_cv, comm=mpi.serial_comm)
        v_R = self.gd.empty()
        v_i = v_R.ravel()
        
        pos_iv = self.gd.get_grid_point_coordinates().reshape((3, -1)).T
        corner_jv = np.dot(np.indices((2, 2, 2)).reshape((3, 8)).T, bigcell_cv)
        for i, pos_v in enumerate(pos_iv):
            r = ((pos_v - corner_jv)**2).sum(axis=1).min()**0.5
            if r == 0:
                v_i[i] = 2 * self.a / pi**0.5
            else:
                v_i[i] = erf(self.a * r) / r
                
        self.K_Q = np.fft.fftn(v_R) * self.gd.dv
        
    def get_potential(self, pd, q_v=None):
        q_c = pd.kd.bzk_kc[0]
        shift_c = (q_c * self.nk_c).round().astype(int)
        max_c = self.gd.N_c // 2
        K_G = pd.zeros()
        N_c = pd.gd.N_c
        for G, Q in enumerate(pd.Q_qG[0]):
            Q_c = (np.unravel_index(Q, N_c) + N_c // 2) % N_c - N_c // 2
            Q_c = Q_c * self.nk_c + shift_c
            if (abs(Q_c) < max_c).all():
                K_G[G] = self.K_Q[tuple(Q_c)]

        qG_Gv = pd.get_reciprocal_vectors(add_q=True)
        if q_v is not None:
            qG_Gv += q_v
        G2_G = np.sum(qG_Gv**2, axis=1)
        #G2_G = pd.G2_qG[0]
        a = self.a
        if pd.kd.gamma:
            K_G[0] += pi / a**2
        else:
            K_G[0] += 4 * pi * (1 - np.exp(-G2_G[0] / (4 * a**2))) / G2_G[0]
        K_G[1:] += 4 * pi * (1 - np.exp(-G2_G[1:] / (4 * a**2))) / G2_G[1:]
        assert pd.dtype == complex
        return K_G



"""This module defines different external potentials."""
import numpy as np

from ase.units import Bohr, Hartree

import _gpaw

__all__ = ['ConstantPotential', 'ConstantElectricField']


def dict2potential(dct):
    """Construct potential from dict."""
    if dct['name'] not in __all__:
        raise ValueError
    return globals()[dct['name']](**dct['kwargs'])
    

class ExternalPotential:
    vext_g = None

    def get_potential(self, gd):
        """Get the potential on a regular 3-d grid.
        
        Will only call calculate_potential() the first time."""
        
        if self.vext_g is None:
            self.calculate_potential(gd)
        return self.vext_g

    def calculate_potential(self, gd):
        raise NotImplementedError
        
    def write(self, writer):
        if hasattr(self, 'todict'):
            from ase.io.jsonio import encode
            writer['ExternalPotential'] = encode(self).replace('"', "'")
        

class ConstantPotential(ExternalPotential):
    """Constant potential for tests."""
    def __init__(self, constant=1.):
        self.constant = constant / Hartree
        
    def __str__(self):
        return 'Constant potential: {0:.3f} eV'.format(self.constant * Hartree)
        
    def calculate_potential(self, gd):
        self.vext_g = gd.zeros() + self.constant

    def todict(self):
        return {'name': 'ConstantPotential',
                'kwargs': {'constant': self.constant * Hartree}}
    
        
class ConstantElectricField(ExternalPotential):
    def __init__(self, strength, direction=[0, 0, 1], tolerance=1e-7):
        """External constant electric field.
        
        strength: float
            Field strength in V/Ang.
        direction: vector
            Polarisation direction.
        """
        d_v = np.asarray(direction)
        self.field_v = strength * d_v / (d_v**2).sum()**0.5 * Bohr / Hartree
        self.tolerance = tolerance
        
    def __str__(self):
        return ('Constant electric field: '
                '({0:.3f}, {1:.3f}, {2:.3f}) eV/Ang'
                .format(*(self.field_v * Hartree / Bohr)))

    def calculate_potential(self, gd):
        d_v = self.field_v / (self.field_v**2).sum()**0.5
        for axis_v in gd.cell_cv[gd.pbc_c]:
            if abs(np.dot(d_v, axis_v)) > self.tolerance:
                raise ValueError(
                    'Field not perpendicular to periodic axis: {0}'
                    .format(axis_v))
                
        center_v = 0.5 * gd.cell_cv.sum(0)
        r_gv = gd.get_grid_point_coordinates().transpose((1, 2, 3, 0))
        self.vext_g = np.dot(r_gv - center_v, self.field_v)
        
    def todict(self):
        strength = (self.field_v**2).sum()**0.5
        return {'name': 'ConstantElectricField',
                'kwargs': {'strength': strength * Hartree / Bohr,
                           'direction': self.field_v / strength}}


class PointChargePotential(ExternalPotential):
    def __init__(self, charges, positions=None,
                 rc=0.2, rc2=np.inf, width=1.0):
        """Point-charge potential.
        
        charges: list of float
            Charges.
        positions: (N, 3)-shaped array-like of float
            Positions of charges in Angstrom.  Can be set later.
        rc: float
            Inner cutoff for Coulomb potential in Angstrom.
        rc2: float
            Outer cutoff for Coulomb potential in Angstrom.
        width: float
            Width for cutoff function for Coulomb part.
            
        For r < rc, 1 / r is replaced by a third order polynomial in r^2 that
        has matching value, first derivative, second derivative and integral.
        
        For rc2 - width < r < rc2, 1 / r is multiplied by a smooth cutoff
        function (a third order polynomium in r).
        
        You can also give rc a negative value.  In that case, this formula
        is used::
            
            (r^4 - rc^4) / (r^5 - |rc|^5)
            
        for all values of r - no cutoff at rc2!
        """
        self.q_p = np.ascontiguousarray(charges, float)
        self.rc = rc / Bohr
        self.rc2 = rc2 / Bohr
        self.width = width / Bohr
        if positions is not None:
            self.set_positions(positions)
        else:
            self.R_pv = None
            
    def __str__(self):
        return ('Point-charge potential '
                '(points: {0}, cutoffs: {1:.3f}, {2:.3f}, {3:.3f} Ang)'
                .format(len(self.q_p),
                        self.rc * Bohr,
                        (self.rc2 - self.width) * Bohr,
                        self.rc2 * Bohr))
            
    def set_positions(self, R_pv):
        """Update positions."""
        self.R_pv = np.asarray(R_pv) / Bohr
        self.vext_g = None
        
    def calculate_potential(self, gd):
        assert gd.orthogonal
        self.vext_g = gd.zeros()
        _gpaw.pc_potential(gd.beg_c, gd.h_cv.diagonal().copy(),
                           self.q_p, self.R_pv, self.rc, self.rc2, self.width,
                           self.vext_g)

    def get_forces(self, calc):
        """Calculate forces from QM charge density on point-charges."""
        dens = calc.density
        F_pv = np.zeros_like(self.R_pv)
        gd = dens.finegd
        _gpaw.pc_potential(gd.beg_c, gd.h_cv.diagonal().copy(),
                           self.q_p, self.R_pv, self.rc, self.rc2, self.width,
                           self.vext_g, dens.rhot_g, F_pv)
        gd.comm.sum(F_pv)
        return F_pv * Hartree / Bohr



import os
import time
from gpaw.io.hdf5_highlevel import File, HyperslabSelection

from gpaw.io import FileReference

import numpy as np

intsize = 4
floatsize = np.array([1], float).itemsize
complexsize = np.array([1], complex).itemsize
itemsizes = {'int': intsize, 'float': floatsize, 'complex': complexsize}


class Writer:
    def __init__(self, name, comm=None):
        self.comm = comm  # for possible future use
        self.dims = {}
        try:
            if self.comm.rank == 0:
                if os.path.isfile(name):
                    os.rename(name, name[:-5] + '.old' + name[-5:])
            self.comm.barrier()
        except AttributeError:
            if os.path.isfile(name):
                os.rename(name, name[:-5] + '.old' + name[-5:])

        if self.comm.size > 1:
            comm = self.comm.get_c_object()
        else:
            comm = None
        self.file = File(name, 'w', comm)
        self.dims_grp = self.file.create_group("Dimensions")
        self.params_grp = self.file.create_group("Parameters")
        self.file.attrs['title'] = 'gpaw_io version="0.1"'
        self.hdf5 = True

    def dimension(self, name, value):
        if name in self.dims.keys() and self.dims[name] != value:
            raise Warning('Dimension %s changed from %s to %s' %
                          (name, self.dims[name], value))
        self.dims[name] = value
        self.dims_grp.attrs[name] = value

    def __setitem__(self, name, value):
        # attributes must be written collectively
        self.params_grp.attrs[name] = value

    def add(self, name, shape, array=None, dtype=None,
            parallel=False, write=True):
        if array is not None:
            array = np.asarray(array)

        # self.dtype, type, itemsize = self.get_data_type(array, dtype)
        if dtype is None:
            self.dtype = array.dtype
        else:
            self.dtype = dtype

        shape = [self.dims[dim] for dim in shape]
        if not shape:
            shape = [1]
        self.dset = self.file.create_dataset(name, shape, self.dtype)
        if array is not None:
            self.fill(array, parallel=parallel, write=write)

    def fill(self, array, *indices, **kwargs):

        parallel = kwargs.pop('parallel', False)
        write = kwargs.pop('write', True)

        if parallel:
            collective = True
        else:
            collective = False

        if not write:
            selection = None
        elif indices:
            selection = HyperslabSelection(indices, self.dset.shape)
        else:
            selection = 'all'
        self.dset.write(array, selection, collective)

    def get_data_type(self, array=None, dtype=None):
        if dtype is None:
            dtype = array.dtype

        if dtype in [int, bool]:
            dtype = np.int32

        dtype = np.dtype(dtype)
        type = {np.int32: 'int',
                np.float64: 'float',
                np.complex128: 'complex'}[dtype.type]

        return dtype, type, dtype.itemsize

    def append(self, name):
        raise NotImplementedError('Append with HDF5 not available.')

    def close(self):
        mtime = int(time.time())
        self.file.attrs['mtime'] = mtime
        self.dims_grp.close()
        self.params_grp.close()
        self.dset.close()
        self.file.close()
        
        
class Reader:
    def __init__(self, name, comm):
        self.comm = comm  # used for broadcasting replicated data
        if self.comm.size > 1:
            comm = self.comm.get_c_object()
        else:
            comm = None
        self.file = File(name, 'r', comm)
        self.dims_grp = self.file['Dimensions']
        self.params_grp = self.file['Parameters']
        self.hdf5 = True

    def dimension(self, name):
        value = self.dims_grp.attrs[name]
        return value

    def __getitem__(self, name):
        value = self.params_grp.attrs[name]
        try:
            value = eval(value, {})
        except (SyntaxError, NameError, TypeError):
            pass
        return value

    def has_array(self, name):
        return name in self.file.keys()
    
    def get(self, name, *indices, **kwargs):

        parallel = kwargs.pop('parallel', False)
        read = kwargs.pop('read', True)
        out = kwargs.pop('out', None)
        broadcast = kwargs.pop('broadcast', False)
        assert not kwargs

        if parallel:
            collective = True
        else:
            collective = False

        dset = self.file[name]
        if indices:
            selection = HyperslabSelection(indices, dset.shape)
            mshape = selection.mshape
        else:
            selection = 'all'
            mshape = dset.shape

        if not read:
            selection = None

        real2complex = False
        if out is None:
            array = np.ndarray(mshape, dset.dtype, order='C')
        elif out.dtype == complex and dset.dtype == float:
            # For real to complex reading i.e TDDFT temporary buffer
            # is also needed
            array = np.ndarray(mshape, dset.dtype, order='C')
            real2complex = True
        else:
            assert isinstance(out, np.ndarray)
            # XXX Check the shapes are compatible
            assert out.shape == mshape
            assert (out.dtype == dset.dtype or
                    (out.dtype == complex and dset.dtype == float))
            array = out

        dset.read(array, selection, collective)

        if real2complex:
            out[:] = array[:]
            array = out

        if broadcast:
            self.comm.broadcast(array, 0)

        if array.shape == ():
            return array.item()
        else:
            return array

    def get_reference(self, name, indices, length=None):
        return HDF5FileReference(self.file, name, indices, length)

    def get_parameters(self):
        return self.params_grp.attrs
   
    def close(self):
        self.file.close()

        
class HDF5FileReference(FileReference):
    def __init__(self, file, name, indices, length):
        self.file = file
        self.name = name
        self.dset = self.file[name]
        self.dtype = self.dset.dtype
        self.shape = self.dset.shape
        self.indices = indices
        self.length = length

    def __len__(self):
        """Length of the first dimension"""
        return self.shape[0]

    def __getitem__(self, indices):
        if not isinstance(indices, tuple):
            indices = (indices,)
        ind = self.indices + indices
        if self.length:
            return self.dset[ind][..., :self.length].copy()
        else:
            return self.dset[ind]



import numpy as np


from ase.units import Bohr, Hartree
from ase import Atoms
from gpaw.tddft import eV_to_aufrequency, aufrequency_to_eV

import gpaw.mpi as mpi
from gpaw.io.tar import Reader, Writer
from gpaw.poisson import PoissonSolver
from gpaw.fd_operators import Gradient
from gpaw.grid_descriptor import GridDescriptor

from gpaw.utilities.extend_grid import extended_grid_descriptor, \
    extend_array, deextend_array, move_atoms


def sendreceive_dict(comm, a_i, dest, b_i, src_i, iitems):
    """
    Send iitems of dictionary b_i distributed according to src_i
    to dictionary a_i in dest.
    """
    requests = []
    for i in iitems:
        # Send and receive
        if comm.rank == dest:
            # Check if dest has it already
            if dest == src_i[i]:
                a_i[i] = b_i[i].copy()
            else:
                requests.append(comm.receive(a_i[i], src_i[i], tag=i,
                                             block=False))
        elif comm.rank == src_i[i]:
            requests.append(comm.send(b_i[i], dest, tag=i, block=False))
    comm.waitall(requests)


class BaseInducedField(object):
    """Partially virtual base class for induced field calculations.
    
    Attributes:
    -----------
    omega_w: ndarray
        Frequencies for Fourier transform in atomic units
    folding: string
        Folding type ('Gauss' or 'Lorentz' or None)
    width: float
        Width parameter for folding
    Frho_wg: ndarray (complex)
        Fourier transform of induced charge density
    Fphi_wg: ndarray (complex)
        Fourier transform of induced electric potential
    Fef_wvg: ndarray (complex)
        Fourier transform of induced electric field
    Ffe_wg: ndarray (float)
        Fourier transform of field enhancement
    Fbgef_v: ndarray (float)
        Fourier transform of background electric field
    """
    
    def __init__(self, filename=None, paw=None, ws='all',
                 frequencies=None, folding='Gauss', width=0.08,
                 readmode=''):
        """
        Parameters:
        -----------
        filename: string
            Filename of a previous ``InducedField`` to be loaded.
            Setting filename disables parameters ``frequencies``,
            ``folding`` and ``width``.
        paw: PAW object
            PAW object for InducedField
        ws: ndarray or list of ints
            Indices for frequencies that are read from given file.
            This parameter is neglected if ``filename`` is not given.
        frequencies: ndarray or list of floats
            Frequencies in eV for Fourier transforms.
            This parameter is neglected if ``filename`` is given.
        folding: string
            Folding type: 'Gauss' or 'Lorentz' or None:
            Gaussian or Lorentzian folding or no folding.
            This parameter is neglected if ``filename`` is given.
        width: float
            Width in eV for the Gaussian (sigma) or Lorentzian (eta) folding
            This parameter is neglected if ``filename`` is given.
        """
        self.dtype = complex

        # These are allocated when calculated
        self.fieldgd = None
        self.Frho_wg = None
        self.Fphi_wg = None
        self.Fef_wvg = None
        self.Ffe_wg = None
        self.Fbgef_v = None

        # has variables
        self.has_paw = False
        self.has_field = False

        if not hasattr(self, 'readwritemode_str_to_list'):
            self.readwritemode_str_to_list = \
                {'': ['Frho', 'atoms'],
                 'all': ['Frho', 'Fphi', 'Fef', 'Ffe', 'atoms'],
                 'field': ['Frho', 'Fphi', 'Fef', 'Ffe', 'atoms']}
 
        if filename is not None:
            self.initialize(paw, allocate=False)
            self.read(filename, ws=ws, mode=readmode)
            return
        
        self.folding = folding
        self.width = width * eV_to_aufrequency
        self.set_folding(folding, width * eV_to_aufrequency)
        
        self.omega_w = np.asarray(frequencies) * eV_to_aufrequency
        self.nw = len(self.omega_w)
        
        self.nv = 3  # dimensionality of the space
        
        self.initialize(paw, allocate=True)
    
    def initialize(self, paw, allocate=True):
        self.allocated = False
        self.has_paw = paw is not None

        if self.has_paw:
            # If no paw is given, then the variables
            # marked with "# !" are created in _read().
            # Other variables are not accessible without
            # paw (TODO: could be implemented...).
            self.paw = paw
            self.world = paw.wfs.world                            # !
            self.domain_comm = paw.wfs.gd.comm                    # !
            self.band_comm = paw.wfs.band_comm                    # !
            self.kpt_comm = paw.wfs.kd.comm                       # !
            self.rank_a = paw.wfs.atom_partition.rank_a
            self.nspins = paw.density.nspins                      # !
            self.setups = paw.wfs.setups
            self.density = paw.density
            self.atoms = paw.atoms                                # !
            self.na = len(self.atoms.get_atomic_numbers())        # !
            self.gd = self.density.gd                             # !
            self.stencil = self.paw.input_parameters.stencils[1]
        
        if allocate:
            self.allocate()
    
    def allocate(self):
        self.allocated = True
    
    def deallocate(self):
        self.fieldgd = None
        self.Frho_wg = None
        self.Fphi_wg = None
        self.Fef_wvg = None
        self.Ffe_wg = None
        self.Fbgef_v = None
        
    def set_folding(self, folding, width):
        """
        width: float
            Width in atomic units
        """
        if width is None:
            folding = None

        self.folding = folding
        if self.folding is None:
            self.width = None
        else:
            self.width = width
        
    def get_induced_density(self, from_density, gridrefinement):
        raise RuntimeError('Virtual member function called')

    def calculate_induced_field(self, from_density='comp',
                                gridrefinement=2,
                                extend_N_cd=None,
                                deextend=False,
                                poisson_nn=3, poisson_relax='J',
                                poisson_eps=1e-20,
                                gradient_n=3):

        if self.has_field and \
           from_density == self.field_from_density and \
           self.Frho_wg is not None:
            Frho_wg = self.Frho_wg
            gd = self.fieldgd
        else:
            Frho_wg, gd = self.get_induced_density(from_density,
                                                   gridrefinement)
        
        # Always extend a bit to get field without jumps
        if extend_N_cd is None:
            extend_N_cd = gradient_n * np.ones(shape=(3, 2), dtype=np.int)
            deextend = True
        
        # Extend grid
        oldgd = gd
        egd, cell_cv, move_c = \
            extended_grid_descriptor(gd, extend_N_cd=extend_N_cd)
        Frho_we = egd.zeros((self.nw,), dtype=self.dtype)
        for w in range(self.nw):
            extend_array(Frho_wg[w], gd, Frho_we[w], egd)
        Frho_wg = Frho_we
        gd = egd
        if not deextend:
            # TODO: this will make atoms unusable with original grid
            self.atoms.set_cell(cell_cv, scale_atoms=False)
            move_atoms(self.atoms, move_c)
        
        # Allocate arrays
        Fphi_wg = gd.zeros((self.nw,), dtype=self.dtype)
        Fef_wvg = gd.zeros((self.nw, self.nv,), dtype=self.dtype)
        Ffe_wg = gd.zeros((self.nw,), dtype=float)
        
        for w in range(self.nw):
            # TODO: better output of progress
            #parprint('%d' % w)
            calculate_field(gd, Frho_wg[w], self.Fbgef_v,
                            Fphi_wg[w], Fef_wvg[w], Ffe_wg[w],
                            nv=self.nv,
                            poisson_nn=poisson_nn,
                            poisson_relax=poisson_relax,
                            poisson_eps=poisson_eps,
                            gradient_n=gradient_n)

        # De-extend grid
        if deextend:
            Frho_wo = oldgd.zeros((self.nw,), dtype=self.dtype)
            Fphi_wo = oldgd.zeros((self.nw,), dtype=self.dtype)
            Fef_wvo = oldgd.zeros((self.nw, self.nv,), dtype=self.dtype)
            Ffe_wo = oldgd.zeros((self.nw,), dtype=float)
            for w in range(self.nw):
                deextend_array(Frho_wo[w], oldgd, Frho_wg[w], gd)
                deextend_array(Fphi_wo[w], oldgd, Fphi_wg[w], gd)
                deextend_array(Ffe_wo[w], oldgd, Ffe_wg[w], gd)
                for v in range(self.nv):
                    deextend_array(Fef_wvo[w][v], oldgd, Fef_wvg[w][v], gd)
            Frho_wg = Frho_wo
            Fphi_wg = Fphi_wo
            Fef_wvg = Fef_wvo
            Ffe_wg = Ffe_wo
            gd = oldgd
    
        # Store results
        self.has_field = True
        self.field_from_density = from_density
        self.fieldgd = gd
        self.Frho_wg = Frho_wg
        self.Fphi_wg = Fphi_wg
        self.Fef_wvg = Fef_wvg
        self.Ffe_wg = Ffe_wg
        
    def _parse_readwritemode(self, mode):
        if isinstance(mode, str):
            try:
                readwrites = self.readwritemode_str_to_list[mode]
            except KeyError:
                raise IOError('unknown readwrite mode string')
        elif isinstance(mode, list):
            readwrites = mode
        else:
            raise IOError('unknown readwrite mode type')
        
        if any(k in readwrites for k in ['Frho', 'Fphi', 'Fef', 'Ffe']):
            readwrites.append('field')

        return readwrites

    def read(self, filename, mode='', ws='all', idiotproof=True):
        if idiotproof and not filename.endswith('.ind'):
            raise IOError('Filename must end with `.ind`.')
        
        reads = self._parse_readwritemode(mode)
        
        # Open reader (handles masters)
        tar = Reader(filename)

        # Actual read
        self.nw = tar.dimension('nw')
        if ws == 'all':
            ws = np.arange(self.nw)
        self.nw = len(ws)
        self._read(tar, reads, ws)

        # Close
        tar.close()
        self.world.barrier()

    def _read(self, tar, reads, ws):
        # Test data type
        dtype = tar['datatype']
        if dtype != self.dtype:
            raise IOError('Data is an incompatible type.')

        # Read dimensions
        na = tar['na']
        self.nv = tar.dimension('nv')
        try:
            nspins = tar.dimension('nspins')
        except KeyError:
            nspins = tar['nspins']
        try:
            ng = np.array((tar.dimension('ng0'),
                           tar.dimension('ng1'),
                           tar.dimension('ng2')))
        except KeyError:
            ng = np.array((tar['ng0'], tar['ng1'], tar['ng2']))

        # Background electric field
        Fbgef_v = tar.get('Fbgef_v')
        
        if self.has_paw:
            # Test dimensions
            if na != self.na:
                raise IOError('natoms is incompatible with calculator')
            if nspins != self.nspins:
                raise IOError('nspins is incompatible with calculator')
            if (ng != self.gd.get_size_of_global_array()).any():
                raise IOError('grid is incompatible with calculator')
            if (Fbgef_v != self.Fbgef_v).any():
                raise IOError('kick is incompatible with calculator')
        else:
            # Construct objects / assign values without paw
            self.na = na
            self.nspins = nspins
            self.Fbgef_v = Fbgef_v

            atomnum_a = tar.get('atomnum_a')
            atompos_av = tar.get('atompos_a')
            atomcell_cv = tar.get('atomcell_cv')
            self.atoms = Atoms(numbers=atomnum_a, positions=atompos_av,
                               cell=atomcell_cv, pbc=False)

            self.world = mpi.world
            self.gd = GridDescriptor(ng + 1, atomcell_cv / Bohr,
                                     pbc_c=False, comm=self.world)
            self.domain_comm = self.gd.comm
            self.band_comm = mpi.SerialCommunicator()
            self.kpt_comm = mpi.SerialCommunicator()

        # Folding
        folding = tar['folding']
        width = tar['width']
        self.set_folding(folding, width)
        
        # Frequencies
        self.omega_w = tar.get('omega_w')[ws]

        # Read field
        if 'field' in reads:
            try:
                nfieldg = np.array((tar.dimension('nfieldg0'),
                                    tar.dimension('nfieldg1'),
                                    tar.dimension('nfieldg2')))
            except KeyError:
                nfieldg = np.array((tar['nfieldg0'],
                                    tar['nfieldg1'],
                                    tar['nfieldg2']))
            self.has_field = True
            self.field_from_density = tar['field_from_density']
            self.fieldgd = self.gd.new_descriptor(N_c=nfieldg + 1)
    
        if 'Frho' in reads:
            self.Frho_wg = self.fieldgd.empty((self.nw), dtype=self.dtype)
            for w, wread in enumerate(ws):
                big_g = tar.get('Frho_wg', wread)
                self.fieldgd.distribute(big_g, self.Frho_wg[w])
        
        if 'Fphi' in reads:
            self.Fphi_wg = self.fieldgd.empty((self.nw), dtype=self.dtype)
            for w, wread in enumerate(ws):
                big_g = tar.get('Fphi_wg', wread)
                self.fieldgd.distribute(big_g, self.Fphi_wg[w])

        if 'Fef' in reads:
            self.Fef_wvg = self.fieldgd.empty((self.nw, self.nv),
                                              dtype=self.dtype)
            for w, wread in enumerate(ws):
                for v in range(self.nv):
                    big_g = tar.get('Fef_wvg', wread, v)
                    self.fieldgd.distribute(big_g, self.Fef_wvg[w][v])

        if 'Ffe' in reads:
            self.Ffe_wg = self.fieldgd.empty((self.nw), dtype=float)
            for w, wread in enumerate(ws):
                big_g = tar.get('Ffe_wg', wread)
                self.fieldgd.distribute(big_g, self.Ffe_wg[w])

    def write(self, filename, mode='', ws='all', idiotproof=True):
        """
        Parameters
        ----------
        mode: string or list of strings
            
        
        """
        if idiotproof and not filename.endswith('.ind'):
            raise IOError('Filename must end with `.ind`.')

        # Masters
        domainmaster = self.domain_comm.rank == 0
        bandmaster = self.band_comm.rank == 0
        kptmaster = self.kpt_comm.rank == 0
        master = domainmaster and bandmaster and kptmaster
#        master = self.world.rank == 0

        if master:
            if self.world.rank != 0:
                raise IOError('master not world master')

        writes = self._parse_readwritemode(mode)

        if ws == 'all':
            ws = np.arange(self.nw)

        if 'field' in writes and self.fieldgd is None:
            raise IOError('field variables cannot be written ' +
                          'before they are calculated')

        # Open writer on master
        if master:
            tar = Writer(filename)
        else:
            tar = None
       
        # Actual write
        self._write(tar, writes, ws,
                    (master, domainmaster, bandmaster, kptmaster))
       
        # Close to flush changes
        if master:
            tar.close()

        # Make sure slaves don't return before master is done
        self.world.barrier()

    def _write(self, tar, writes, ws, masters):
        master, domainmaster, bandmaster, kptmaster = masters

        nw = len(ws)

        if master:
            # Write parameters/dimensions
            tar['datatype'] = {float: 'float', complex: 'complex'}[self.dtype]
            tar['folding'] = self.folding
            tar['width'] = self.width
            
            tar['na'] = self.na
            tar['nspins'] = self.nspins
            tar.dimension('nv', self.nv)
            tar.dimension('nw', nw)
            tar.dimension('nspins', self.nspins)
            
            # Write grid
            ng = self.gd.get_size_of_global_array()
            tar['ng0'] = ng[0]
            tar['ng1'] = ng[1]
            tar['ng2'] = ng[2]
            tar.dimension('ng0', ng[0])
            tar.dimension('ng1', ng[1])
            tar.dimension('ng2', ng[2])

            # Write field grid
            if 'field' in writes:
                nfieldg = self.fieldgd.get_size_of_global_array()
                tar['nfieldg0'] = nfieldg[0]
                tar['nfieldg1'] = nfieldg[1]
                tar['nfieldg2'] = nfieldg[2]
                tar.dimension('nfieldg0', nfieldg[0])
                tar.dimension('nfieldg1', nfieldg[1])
                tar.dimension('nfieldg2', nfieldg[2])
                tar['field_from_density'] = self.field_from_density

            # Write frequencies
            tar.add('omega_w', ('nw',), self.omega_w[ws], dtype=float)

            # Write background electric field
            tar.add('Fbgef_v', ('nv',), self.Fbgef_v, dtype=float)
       
            # Write system description
            # TODO: remove this and use ASE's atoms object instead
            # if 'atoms' in writes:
            if True:  # Always write atoms!
                atomnum_a = self.atoms.get_atomic_numbers()
                atompos_a = self.atoms.get_positions()
                atomcell_cv = self.atoms.get_cell()
                
                tar.dimension('na', self.na)
                tar.add('atomnum_a', ('na',), atomnum_a, dtype=int)
                tar.add('atompos_a', ('na', 'nv',), atompos_a, dtype=float)
                tar.add('atomcell_cv', ('nv', 'nv',), atomcell_cv, dtype=float)
                
        if 'Frho' in writes:
            if master:
                tar.add('Frho_wg',
                        ('nw', 'nfieldg0', 'nfieldg1', 'nfieldg2', ),
                        dtype=self.dtype)
            for w in ws:
                big_g = self.fieldgd.collect(self.Frho_wg[w])
                if master:
                    tar.fill(big_g)

        if 'Fphi' in writes:
            if master:
                tar.add('Fphi_wg',
                        ('nw', 'nfieldg0', 'nfieldg1', 'nfieldg2', ),
                        dtype=self.dtype)
            for w in ws:
                big_g = self.fieldgd.collect(self.Fphi_wg[w])
                if master:
                    tar.fill(big_g)

        if 'Fef' in writes:
            if master:
                tar.add('Fef_wvg',
                        ('nw', 'nv', 'nfieldg0', 'nfieldg1', 'nfieldg2', ),
                        dtype=self.dtype)
            for w in ws:
                for v in range(self.nv):
                    big_g = self.fieldgd.collect(self.Fef_wvg[w][v])
                    if master:
                        tar.fill(big_g)
                        
        if 'Ffe' in writes:
            if master:
                tar.add('Ffe_wg',
                        ('nw', 'nfieldg0', 'nfieldg1', 'nfieldg2', ),
                        dtype=float)
            for w in ws:
                big_g = self.fieldgd.collect(self.Ffe_wg[w])
                if master:
                    tar.fill(big_g)


def calculate_field(gd, rho_g, bgef_v,
                    phi_g, ef_vg, fe_g,  # preallocated numpy arrays
                    nv=3, poisson_nn=3, poisson_relax='J',
                    gradient_n=3, poisson_eps=1e-20):
    
    dtype = rho_g.dtype
    yes_complex = dtype == complex

    phi_g[:] = 0.0
    ef_vg[:] = 0.0
    fe_g[:] = 0.0
    tmp_g = gd.zeros(dtype=float)

    # Poissonsolver
    poissonsolver = PoissonSolver(nn=poisson_nn,
                                  relax=poisson_relax,
                                  eps=poisson_eps)
    poissonsolver.set_grid_descriptor(gd)
    poissonsolver.initialize()

    # Potential, real part
    poissonsolver.solve(tmp_g, rho_g.real.copy())
    phi_g += tmp_g
    # Potential, imag part
    if yes_complex:
        tmp_g[:] = 0.0
        poissonsolver.solve(tmp_g, rho_g.imag.copy())
        phi_g += 1.0j * tmp_g
    
    # Gradient
    gradient = [Gradient(gd, v, scale=1.0, n=gradient_n)
                for v in range(nv)]
    for v in range(nv):
        # Electric field, real part
        gradient[v].apply(-phi_g.real, tmp_g)
        ef_vg[v] += tmp_g
        # Electric field, imag part
        if yes_complex:
            gradient[v].apply(-phi_g.imag, tmp_g)
            ef_vg[v] += 1.0j * tmp_g
    
    # Electric field enhancement
    tmp_g[:] = 0.0  # total electric field norm
    bgefnorm = 0.0  # background electric field norm
    for v in range(nv):
        tmp_g += np.absolute(bgef_v[v] + ef_vg[v])**2
        bgefnorm += np.absolute(bgef_v[v])**2
        
    tmp_g = np.sqrt(tmp_g)
    bgefnorm = np.sqrt(bgefnorm)

    fe_g[:] = tmp_g / bgefnorm


def zero_pad(a):
    """Add zeros to both sides of all axis on array a.
    
    Not parallel safe.
    """
    
    z_shape = np.zeros(shape=len(a.shape))
    z_shape[-3:] = 2
    b_shape = np.array(a.shape) + z_shape
    
    b = np.zeros(shape=b_shape, dtype=a.dtype)
    
    b[..., 1:-1, 1:-1, 1:-1] = a
    return b


def read_data(filename, keys=None, ws='all'):
    """
    Read data arrays for post processing.
    
    Not parallel safe. No GridDescriptor, only numpy arrays.
    
    Parameters
    ----------
    filename: string
        File to be read.
    keys: list of strings
        Keys to be read.
    ws: list of ints
        Indices of frequencies to be read.
    """
    
    key_to_tarname = {'n0t_sG': 'n0t_sG',
                      'Fnt_wsG': 'Fnt_wsG',
                      'Frho_wg': 'Frho_wg',
                      'Fphi_wg': 'Fphi_wg',
                      'Fef_wvg': 'Fef_wvg',
                      'Ffe_wg': 'Ffe_wg',
                      'eps0_G': 'eps0_G'
                      }
    
    print('Reading %s' % (filename))
    
    if keys is None:
        keys = key_to_tarname.keys()  # all keys
    
    tar = Reader(filename)

    omega_w = tar.get('omega_w')

    if ws == 'all':
        ws = range(len(omega_w))
    else:
        omega_w = omega_w[ws]
    
    freq_w = omega_w * aufrequency_to_eV
    
    try:
        nspins = tar.dimension('nspins')
    except KeyError:
        nspins = None
    
    na = tar['na']
    try:
        atomnum_a = tar.get('atomnum_a')
        atompos_av = tar.get('atompos_a')
        atomcell_cv = tar.get('atomcell_cv')
        Fbgef_v = tar.get('Fbgef_v')
        
        atom_a = []
        for a in range(na):
            atom_a.append({'atom': atomnum_a[a], 'pos': atompos_av[a]})
    except KeyError:
        atom_a = None
        atomcell_cv = None
        Fbgef_v = None
        print('no atoms')
    
    data = dict()
    data['freq_w'] = freq_w
    data['nspins'] = nspins
    data['na'] = na
    data['atom_a'] = atom_a
    data['cell_cv'] = atomcell_cv
    data['Fbgef_v'] = Fbgef_v
   
    try:
        data['corner1_v'] = tar.get('corner1_v')
        data['corner2_v'] = tar.get('corner2_v')
    except:
        print('no corners')

    try:
        FD_awsp = {}
        D0_asp = {}
        for a in range(na):
            FD_awsp[a] = tar.get('FD_%dwsp' % a)
            D0_asp[a] = tar.get('D0_%dsp' % a)
        data['FD_awsp'] = FD_awsp
        data['D0_asp'] = D0_asp
    except KeyError:
        print('no FD_awsp')

    for key in keys:
        try:
            if '_w' in key:
                tmp = zero_pad(tar.get(key_to_tarname[key], ws[0]))
                data[key] = np.empty((len(ws),) + tmp.shape, tmp.dtype)
                data[key][0] = tmp
                for w, wread in enumerate(ws[1:], 1):
                    data[key][w] = zero_pad(tar.get(key_to_tarname[key],
                                                    wread))
            else:
                data[key] = zero_pad(tar.get(key_to_tarname[key]))
        except KeyError:
            print('no %s' % key)
            pass

    tar.close()
    
    return data


# TOOD: remove/edit this function
def calculate_oscstr(Fn_wg, omega_w, box, kick):
    omega_w = np.array(omega_w)
    omega_w *= eV_to_aufrequency  # to a.u.
    box = box.copy() / Bohr  # to a.u
    volume = box.prod()
    ng = np.array(Fn_wg[0].shape)
    dV = volume / (ng - 1).prod()  # Note: data is zeropadded to both sides
    
    Fn_wg_im = Fn_wg.imag
    if kick[0] != 0.0:
        osc_w = -2 * omega_w / np.pi / kick[0] * \
            ((Fn_wg_im.sum(axis=2)).sum(axis=2) *
             np.linspace(0, box[0], ng[0])).sum(axis=1) * dV
    elif kick[1] != 0.0:
        osc_w = -2 * omega_w / np.pi / kick[1] * \
            ((Fn_wg_im.sum(axis=1)).sum(axis=2) *
             np.linspace(0, box[1], ng[1])).sum(axis=1) * dV
    elif kick[2] != 0.0:
        osc_w = -2 * omega_w / np.pi / kick[2] * \
            ((Fn_wg_im.sum(axis=1)).sum(axis=1) *
             np.linspace(0, box[2], ng[2])).sum(axis=1) * dV
    return osc_w / Hartree  # to 1/eV


# TOOD: remove/edit this function
def calculate_polarizability(Fn_wg, box, kick):
    box = box.copy() / Bohr  # to a.u
    volume = box.prod()
    ng = np.array(Fn_wg[0].shape)
    dV = volume / (ng - 1).prod()  # Note: data is zeropadded to both sides
    
    if kick[0] != 0.0:
        pol_w = -1.0 / kick[0] * \
            ((Fn_wg.sum(axis=2)).sum(axis=2) *
             np.linspace(0, box[0], ng[0])).sum(axis=1) * dV
    elif kick[1] != 0.0:
        pol_w = -1.0 / kick[1] * \
            ((Fn_wg.sum(axis=1)).sum(axis=2) *
             np.linspace(0, box[1], ng[1])).sum(axis=1) * dV
    elif kick[2] != 0.0:
        pol_w = -1.0 / kick[2] * \
            ((Fn_wg.sum(axis=1)).sum(axis=1) *
             np.linspace(0, box[2], ng[2])).sum(axis=1) * dV
    return pol_w / Hartree**2  # to 1/eV**2



from gpaw.atom.atompaw import AtomPAW
from gpaw.upf import UPFSetupData

def check(title, calc, epsref_n, threshold):
    eps_n = calc.wfs.kpt_u[0].eps_n
    print(title)
    for i, epsref in enumerate(epsref_n):
        err = abs(epsref - eps_n[i])
        ok = (err <= threshold)
        status = 'ok' if ok else 'FAILED'
        print('state %d | eps=%f | ref=%f | err=%f | tol=%s | %s' % (i,
                                                                     eps_n[i],
                                                                     epsref,
                                                                     err,
                                                                     threshold,
                                                                     status))
        assert ok

kwargs = dict(txt=None)

for setup in ['paw', 'hgh', UPFSetupData('H.pz-hgh.UPF')]:
    calc = AtomPAW('H', [[[1]]],
                   rcut=12.0, h=0.05,
                   setups={'H': setup}, **kwargs)
    tol = 5e-4 if setup in ['paw', 'hgh'] else 1e-3 # horrible UPF right now
    check('H %s' % setup, calc, [-0.233471], tol)


for setup in ['paw', 'hgh', UPFSetupData('O.pz-hgh.UPF')]:
    calc = AtomPAW('O', [[[2], [4]]],
                   rcut=10.0, h=0.035,
                   setups={'O': setup}, **kwargs)
    tol = 1e-3 if setup in ['paw', 'hgh'] else 5e-3 # horrible UPF right now
    check('O %s' % setup, calc, [-0.871362, -0.338381], tol)



# Copyright (C) 2010  CAMd
# Please see the accompanying LICENSE file for further information.

"""This module provides all the classes and functions associated with the
evaluation of exact exchange with k-point sampling."""
from __future__ import print_function
from time import time
from math import pi, sqrt

import numpy as np
from ase.units import Hartree
from ase.dft.kpoints import monkhorst_pack
from ase.utils.timing import Timer

import gpaw.mpi as mpi
import gpaw.fftw as fftw
from gpaw.xc.hybrid import HybridXCBase
from gpaw.kpt_descriptor import KPointDescriptor
from gpaw.wavefunctions.pw import PWDescriptor, PWLFC
from gpaw.utilities import pack, unpack2, packed_index, logfile, erf
from gpaw.utilities.ewald import madelung


class HybridXC(HybridXCBase):
    orbital_dependent = True

    def __init__(self, name, hybrid=None, xc=None,
                 alpha=None,
                 gamma_point=1,
                 method='standard',
                 bandstructure=False,
                 logfilename='-', bands=None,
                 fcut=1e-10,
                 molecule=False,
                 qstride=1,
                 world=None):
        """Mix standard functionals with exact exchange.

        name: str
            Name of functional: EXX, PBE0, HSE03, HSE06
        hybrid: float
            Fraction of exact exchange.
        xc: str or XCFunctional object
            Standard DFT functional with scaled down exchange.
        method: str
            Use 'standard' standard formula and 'acdf for
            adiabatic-connection dissipation fluctuation formula.
        alpha: float
            XXX describe
        gamma_point: bool
            0: Skip k2-k1=0 interactions.
            1: Use the alpha method.
            2: Integrate the gamma point.
        bandstructure: bool
            Calculate bandstructure instead of just the total energy.
        bands: list of int
            List of bands to calculate bandstructure for.  Default is
            all bands.
        molecule: bool
            Decouple electrostatic interactions between periodically
            repeated images.
        fcut: float
            Threshold for empty band.
        """

        self.alpha = alpha
        self.fcut = fcut

        self.gamma_point = gamma_point
        self.method = method
        self.bandstructure = bandstructure
        self.bands = bands

        self.fd = logfilename
        self.write_timing_information = True

        HybridXCBase.__init__(self, name, hybrid, xc)

        # EXX energies:
        self.exx = None  # total
        self.evv = None  # valence-valence (pseudo part)
        self.evvacdf = None  # valence-valence (pseudo part)
        self.devv = None  # valence-valence (PAW correction)
        self.evc = None  # valence-core
        self.ecc = None  # core-core

        self.exx_skn = None  # bandstructure

        self.qlatest = None

        if world is None:
            world = mpi.world
        self.world = world

        self.molecule = molecule
        
        if isinstance(qstride, int):
            qstride = [qstride] * 3
        self.qstride_c = np.asarray(qstride)
        
        self.timer = Timer()

    def log(self, *args, **kwargs):
        print(file=self.fd, *args, **kwargs)
        self.fd.flush()

    def calculate_radial(self, rgd, n_sLg, Y_L, v_sg,
                         dndr_sLg=None, rnablaY_Lv=None,
                         tau_sg=None, dedtau_sg=None):
        return self.xc.calculate_radial(rgd, n_sLg, Y_L, v_sg,
                                        dndr_sLg, rnablaY_Lv)
    
    def calculate_paw_correction(self, setup, D_sp, dEdD_sp=None,
                                 addcoredensity=True, a=None):
        return self.xc.calculate_paw_correction(setup, D_sp, dEdD_sp,
                                 addcoredensity, a)
    
    def initialize(self, dens, ham, wfs, occupations):
        assert wfs.bd.comm.size == 1

        self.xc.initialize(dens, ham, wfs, occupations)

        self.dens = dens
        self.wfs = wfs

        # Make a k-point descriptor that is not distributed
        # (self.kd.comm is serial_comm):
        self.kd = wfs.kd.copy()

        self.fd = logfile(self.fd, self.world.rank)

        wfs.initialize_wave_functions_from_restart_file()

    def set_positions(self, spos_ac):
        self.spos_ac = spos_ac

    def calculate(self, gd, n_sg, v_sg=None, e_g=None):
        # Normal XC contribution:
        exc = self.xc.calculate(gd, n_sg, v_sg, e_g)

        # Add EXX contribution:
        return exc + self.exx * self.hybrid

    def calculate_exx(self):
        """Non-selfconsistent calculation."""

        self.timer.start('EXX')
        self.timer.start('Initialization')
        
        kd = self.kd
        wfs = self.wfs

        if fftw.FFTPlan is fftw.NumpyFFTPlan:
            self.log('NOT USING FFTW !!')

        self.log('Spins:', self.wfs.nspins)

        W = max(1, self.wfs.kd.comm.size // self.wfs.nspins)
        # Are the k-points distributed?
        kparallel = (W > 1)

        # Find number of occupied bands:
        self.nocc_sk = np.zeros((self.wfs.nspins, kd.nibzkpts), int)
        for kpt in self.wfs.kpt_u:
            for n, f in enumerate(kpt.f_n):
                if abs(f) < self.fcut:
                    self.nocc_sk[kpt.s, kpt.k] = n
                    break
            else:
                self.nocc_sk[kpt.s, kpt.k] = self.wfs.bd.nbands
        self.wfs.kd.comm.sum(self.nocc_sk)

        noccmin = self.nocc_sk.min()
        noccmax = self.nocc_sk.max()
        self.log('Number of occupied bands (min, max): %d, %d' %
                 (noccmin, noccmax))
        
        self.log('Number of valence electrons:', self.wfs.setups.nvalence)

        if self.bandstructure:
            self.log('Calculating eigenvalue shifts.')

            # allocate array for eigenvalue shifts:
            self.exx_skn = np.zeros((self.wfs.nspins,
                                     kd.nibzkpts,
                                     self.wfs.bd.nbands))

            if self.bands is None:
                noccmax = self.wfs.bd.nbands
            else:
                noccmax = max(max(self.bands) + 1, noccmax)

        N_c = self.kd.N_c

        vol = wfs.gd.dv * wfs.gd.N_c.prod()
        if self.alpha is None:
            alpha = 6 * vol**(2 / 3.0) / pi**2
        else:
            alpha = self.alpha
        if self.gamma_point == 1:
            if alpha == 0.0:
                qvol = (2 * np.pi)**3 / vol / N_c.prod()
                self.gamma = 4 * np.pi * (3 * qvol / (4 * np.pi))**(1 / 3.) / qvol
            else:
                self.gamma = self.calculate_gamma(vol, alpha)
        else:
            kcell_cv = wfs.gd.cell_cv.copy()
            kcell_cv[0] *= N_c[0]
            kcell_cv[1] *= N_c[1]
            kcell_cv[2] *= N_c[2]
            self.gamma = madelung(kcell_cv) * vol * N_c.prod() / (4 * np.pi)

        self.log('Value of alpha parameter: %.3f Bohr^2' % alpha)
        self.log('Value of gamma parameter: %.3f Bohr^2' % self.gamma)
            
        # Construct all possible q=k2-k1 vectors:
        Nq_c = (N_c - 1) // self.qstride_c
        i_qc = np.indices(Nq_c * 2 + 1, float).transpose(
            (1, 2, 3, 0)).reshape((-1, 3))
        self.bzq_qc = (i_qc - Nq_c) / N_c * self.qstride_c
        self.q0 = ((Nq_c * 2 + 1).prod() - 1) // 2  # index of q=(0,0,0)
        assert not self.bzq_qc[self.q0].any()

        # Count number of pairs for each q-vector:
        self.npairs_q = np.zeros(len(self.bzq_qc), int)
        for s in range(kd.nspins):
            for k1 in range(kd.nibzkpts):
                for k2 in range(kd.nibzkpts):
                    for K2, q, n1_n, n2 in self.indices(s, k1, k2):
                        self.npairs_q[q] += len(n1_n)

        self.npairs0 = self.npairs_q.sum()  # total number of pairs

        self.log('Number of pairs:', self.npairs0)

        # Distribute q-vectors to Q processors:
        Q = self.world.size // self.wfs.kd.comm.size
        myrank = self.world.rank // self.wfs.kd.comm.size
        rank = 0
        N = 0
        myq = []
        nq = 0
        for q, n in enumerate(self.npairs_q):
            if n > 0:
                nq += 1
                if rank == myrank:
                    myq.append(q)
            N += n
            if N >= (rank + 1.0) * self.npairs0 / Q:
                rank += 1

        assert len(myq) > 0, 'Too few q-vectors for too many processes!'
        self.bzq_qc = self.bzq_qc[myq]
        try:
            self.q0 = myq.index(self.q0)
        except ValueError:
            self.q0 = None

        self.log('%d x %d x %d k-points' % tuple(self.kd.N_c))
        self.log('Distributing %d IBZ k-points over %d process(es).' %
                 (kd.nibzkpts, self.wfs.kd.comm.size))
        self.log('Distributing %d q-vectors over %d process(es).' % (nq, Q))

        # q-point descriptor for my q-vectors:
        qd = KPointDescriptor(self.bzq_qc)

        # Plane-wave descriptor for all wave-functions:
        self.pd = PWDescriptor(wfs.pd.ecut, wfs.gd,
                               dtype=wfs.pd.dtype, kd=kd)

        # Plane-wave descriptor pair-densities:
        self.pd2 = PWDescriptor(self.dens.pd2.ecut, self.dens.gd,
                                dtype=wfs.dtype, kd=qd)

        self.log('Cutoff energies:')
        self.log('    Wave functions:       %10.3f eV' %
                 (self.pd.ecut * Hartree))
        self.log('    Density:              %10.3f eV' %
                 (self.pd2.ecut * Hartree))

        # Calculate 1/|G+q|^2 with special treatment of |G+q|=0:
        G2_qG = self.pd2.G2_qG
        if self.q0 is None:
            if self.omega is None:
                self.iG2_qG = [1.0 / G2_G for G2_G in G2_qG]
            else:
                self.iG2_qG = [(1.0 / G2_G *
                                (1 - np.exp(-G2_G / (4 * self.omega**2))))
                               for G2_G in G2_qG]
        else:
            G2_qG[self.q0][0] = 117.0  # avoid division by zero
            if self.omega is None:
                self.iG2_qG = [1.0 / G2_G for G2_G in G2_qG]
                self.iG2_qG[self.q0][0] = self.gamma
            else:
                self.iG2_qG = [(1.0 / G2_G *
                                (1 - np.exp(-G2_G / (4 * self.omega**2))))
                               for G2_G in G2_qG]
                self.iG2_qG[self.q0][0] = 1 / (4 * self.omega**2)
            G2_qG[self.q0][0] = 0.0  # restore correct value

        # Compensation charges:
        self.ghat = PWLFC([setup.ghat_l for setup in wfs.setups], self.pd2)
        self.ghat.set_positions(self.spos_ac)

        if self.molecule:
            self.initialize_gaussian()
            self.log('Value of beta parameter: %.3f 1/Bohr^2' % self.beta)
            
        self.timer.stop('Initialization')
        
        # Ready ... set ... go:
        self.t0 = time()
        self.npairs = 0
        self.evv = 0.0
        self.evvacdf = 0.0
        for s in range(self.wfs.nspins):
            kpt1_q = [KPoint(self.wfs, noccmax).initialize(kpt)
                      for kpt in self.wfs.kpt_u if kpt.s == s]
            kpt2_q = kpt1_q[:]

            if len(kpt1_q) == 0:
                # No s-spins on this CPU:
                continue

            # Send and receive ranks:
            srank = self.wfs.kd.get_rank_and_index(
                s, (kpt1_q[0].k - 1) % kd.nibzkpts)[0]
            rrank = self.wfs.kd.get_rank_and_index(
                s, (kpt1_q[-1].k + 1) % kd.nibzkpts)[0]

            # Shift k-points kd.nibzkpts - 1 times:
            for i in range(kd.nibzkpts):
                if i < kd.nibzkpts - 1:
                    if kparallel:
                        kpt = kpt2_q[-1].next(self.wfs)
                        kpt.start_receiving(rrank)
                        kpt2_q[0].start_sending(srank)
                    else:
                        kpt = kpt2_q[0]

                self.timer.start('Calculate')
                for kpt1, kpt2 in zip(kpt1_q, kpt2_q):
                    # Loop over all k-points that k2 can be mapped to:
                    for K2, q, n1_n, n2 in self.indices(s, kpt1.k, kpt2.k):
                        self.apply(K2, q, kpt1, kpt2, n1_n, n2)
                self.timer.stop('Calculate')

                if i < kd.nibzkpts - 1:
                    self.timer.start('Wait')
                    if kparallel:
                        kpt.wait()
                        kpt2_q[0].wait()
                    self.timer.stop('Wait')
                    kpt2_q.pop(0)
                    kpt2_q.append(kpt)

        self.evv = self.world.sum(self.evv)
        self.evvacdf = self.world.sum(self.evvacdf)
        self.calculate_exx_paw_correction()
        
        if self.method == 'standard':
            self.exx = self.evv + self.devv + self.evc + self.ecc
        elif self.method == 'acdf':
            self.exx = self.evvacdf + self.devv + self.evc + self.ecc
        else:
            1 / 0

        self.log('Exact exchange energy:')
        for txt, e in [
            ('core-core', self.ecc),
            ('valence-core', self.evc),
            ('valence-valence (pseudo, acdf)', self.evvacdf),
            ('valence-valence (pseudo, standard)', self.evv),
            ('valence-valence (correction)', self.devv),
            ('total (%s)' % self.method, self.exx)]:
            self.log('    %-36s %14.6f eV' % (txt + ':', e * Hartree))

        self.log('Total time: %10.3f seconds' % (time() - self.t0))

        self.npairs = self.world.sum(self.npairs)
        assert self.npairs == self.npairs0
        
        self.timer.stop('EXX')
        self.timer.write(self.fd)

    def calculate_gamma(self, vol, alpha):
        if self.molecule:
            return 0.0

        N_c = self.kd.N_c
        offset_c = (N_c + 1) % 2 * 0.5 / N_c
        bzq_qc = monkhorst_pack(N_c) + offset_c
        qd = KPointDescriptor(bzq_qc)
        pd = PWDescriptor(self.wfs.pd.ecut, self.wfs.gd, kd=qd)
        gamma = (vol / (2 * pi)**2 * sqrt(pi / alpha) *
                 self.kd.nbzkpts)
        for G2_G in pd.G2_qG:
            if G2_G[0] < 1e-7:
                G2_G = G2_G[1:]
            gamma -= np.dot(np.exp(-alpha * G2_G), G2_G**-1)
        return gamma / self.qstride_c.prod()

    def indices(self, s, k1, k2):
        """Generator for (K2, q, n1, n2) indices for (k1, k2) pair.

        s: int
            Spin index.
        k1: int
            Index of k-point in the IBZ.
        k2: int
            Index of k-point in the IBZ.

        Returns (K, q, n1_n, n2), where K then index of the k-point in
        the BZ that k2 is mapped to, q is the index of the q-vector
        between K and k1, and n1_n is a list of bands that should be
        combined with band n2."""

        for K, k in enumerate(self.kd.bz2ibz_k):
            if k == k2:
                for K, q, n1_n, n2 in self._indices(s, k1, k2, K):
                    yield K, q, n1_n, n2
            
    def _indices(self, s, k1, k2, K2):
        k1_c = self.kd.ibzk_kc[k1]
        k2_c = self.kd.bzk_kc[K2]
        q_c = k2_c - k1_c
        q = abs(self.bzq_qc - q_c).sum(1).argmin()
        if abs(self.bzq_qc[q] - q_c).sum() > 1e-7:
            return

        if self.gamma_point == 0 and q == self.q0:
            return

        nocc1 = self.nocc_sk[s, k1]
        nocc2 = self.nocc_sk[s, k2]

        # Is k2 in the IBZ?
        is_ibz2 = (self.kd.ibz2bz_k[k2] == K2)

        for n2 in range(self.wfs.bd.nbands):
            # Find range of n1's (from n1a to n1b-1):
            if is_ibz2:
                # We get this combination twice, so let's only do half:
                if k1 >= k2:
                    n1a = n2
                else:
                    n1a = n2 + 1
            else:
                n1a = 0

            n1b = self.wfs.bd.nbands

            if self.bandstructure:
                if n2 >= nocc2:
                    n1b = min(n1b, nocc1)
            else:
                if n2 >= nocc2:
                    break
                n1b = min(n1b, nocc1)

            if self.bands is not None:
                assert self.bandstructure
                n1_n = []
                for n1 in range(n1a, n1b):
                    if (n1 in self.bands and n2 < nocc2 or
                        is_ibz2 and n2 in self.bands and n1 < nocc1):
                        n1_n.append(n1)
                n1_n = np.array(n1_n)
            else:
                n1_n = np.arange(n1a, n1b)

            if len(n1_n) == 0:
                continue

            yield K2, q, n1_n, n2

    def apply(self, K2, q, kpt1, kpt2, n1_n, n2):
        k20_c = self.kd.ibzk_kc[kpt2.k]
        k2_c = self.kd.bzk_kc[K2]

        if k2_c.any():
            self.timer.start('Initialize plane waves')
            eik2r_R = self.wfs.gd.plane_wave(k2_c)
            eik20r_R = self.wfs.gd.plane_wave(k20_c)
            self.timer.stop('Initialize plane waves')
        else:
            eik2r_R = 1.0
            eik20r_R = 1.0

        w1 = self.kd.weight_k[kpt1.k]
        w2 = self.kd.weight_k[kpt2.k]

        # Is k2 in the 1. BZ?
        is_ibz2 = (self.kd.ibz2bz_k[kpt2.k] == K2)

        e_n = self.calculate_interaction(n1_n, n2, kpt1, kpt2, q, K2,
                                         eik20r_R, eik2r_R,
                                         is_ibz2)

        e_n *= 1.0 / self.kd.nbzkpts / self.wfs.nspins * self.qstride_c.prod()
        
        if q == self.q0:
            e_n[n1_n == n2] *= 0.5

        f1_n = kpt1.f_n[n1_n]
        eps1_n = kpt1.eps_n[n1_n]
        f2 = kpt2.f_n[n2]
        eps2 = kpt2.eps_n[n2]

        s_n = np.sign(eps2 - eps1_n)

        evv = (f1_n * f2 * e_n).sum()
        evvacdf = 0.5 * (f1_n * (1 - s_n) * e_n +
                         f2 * (1 + s_n) * e_n).sum()
        self.evv += evv * w1
        self.evvacdf += evvacdf * w1
        if is_ibz2:
            self.evv += evv * w2
            self.evvacdf += evvacdf * w2

        if self.bandstructure:
            x = self.wfs.nspins
            self.exx_skn[kpt1.s, kpt1.k, n1_n] += x * f2 * e_n
            if is_ibz2:
                self.exx_skn[kpt2.s, kpt2.k, n2] += x * np.dot(f1_n, e_n)

    def calculate_interaction(self, n1_n, n2, kpt1, kpt2, q, k,
                              eik20r_R, eik2r_R, is_ibz2):
        """Calculate Coulomb interactions.

        For all n1 in the n1_n list, calculate interaction with n2."""

        # number of plane waves:
        ng1 = self.wfs.ng_k[kpt1.k]
        ng2 = self.wfs.ng_k[kpt2.k]

        # Transform to real space and apply symmetry operation:
        self.timer.start('IFFT1')
        if is_ibz2:
            u2_R = self.pd.ifft(kpt2.psit_nG[n2, :ng2], kpt2.k)
        else:
            psit2_R = self.pd.ifft(kpt2.psit_nG[n2, :ng2], kpt2.k) * eik20r_R
            self.timer.start('Symmetry transform')
            u2_R = self.kd.transform_wave_function(psit2_R, k) / eik2r_R
            self.timer.stop()
        self.timer.stop()

        # Calculate pair densities:
        nt_nG = self.pd2.zeros(len(n1_n), q=q)
        for n1, nt_G in zip(n1_n, nt_nG):
            self.timer.start('IFFT2')
            u1_R = self.pd.ifft(kpt1.psit_nG[n1, :ng1], kpt1.k)
            self.timer.stop()
            nt_R = u1_R.conj() * u2_R
            self.timer.start('FFT')
            nt_G[:] = self.pd2.fft(nt_R, q)
            self.timer.stop()
        
        s = self.kd.sym_k[k]
        time_reversal = self.kd.time_reversal_k[k]
        k2_c = self.kd.ibzk_kc[kpt2.k]

        self.timer.start('Compensation charges')
        Q_anL = {}  # coefficients for shape functions
        for a, P1_ni in kpt1.P_ani.items():
            P1_ni = P1_ni[n1_n]

            if is_ibz2:
                P2_i = kpt2.P_ani[a][n2]
            else:
                b = self.kd.symmetry.a_sa[s, a]
                S_c = (np.dot(self.spos_ac[a], self.kd.symmetry.op_scc[s]) -
                       self.spos_ac[b])
                assert abs(S_c.round() - S_c).max() < 1e-5
                if self.ghat.dtype == complex:
                    x = np.exp(2j * pi * np.dot(k2_c, S_c))
                else:
                    x = 1.0
                P2_i = np.dot(self.wfs.setups[a].R_sii[s],
                              kpt2.P_ani[b][n2]) * x
                if time_reversal:
                    P2_i = P2_i.conj()

            D_np = []
            for P1_i in P1_ni:
                D_ii = np.outer(P1_i.conj(), P2_i)
                D_np.append(pack(D_ii))
            Q_anL[a] = np.dot(D_np, self.wfs.setups[a].Delta_pL)
            
        self.timer.start('Expand')
        if q != self.qlatest:
            self.f_IG = self.ghat.expand(q)
            self.qlatest = q
        self.timer.stop('Expand')

        # Add compensation charges:
        self.ghat.add(nt_nG, Q_anL, q, self.f_IG)
        self.timer.stop('Compensation charges')

        if self.molecule and n2 in n1_n:
            nn = (n1_n == n2).nonzero()[0][0]
            nt_nG[nn] -= self.ngauss_G
        else:
            nn = None
            
        iG2_G = self.iG2_qG[q]
        
        # Calculate energies:
        e_n = np.empty(len(n1_n))
        for n, nt_G in enumerate(nt_nG):
            e_n[n] = -4 * pi * np.real(self.pd2.integrate(nt_G, nt_G * iG2_G))
            self.npairs += 1
        
        if nn is not None:
            e_n[nn] -= 2 * (self.pd2.integrate(nt_nG[nn], self.vgauss_G) +
                            (self.beta / 2 / pi)**0.5)

        if self.write_timing_information:
            t = (time() - self.t0) / len(n1_n)
            self.log('Time for first pair-density: %10.3f seconds' % t)
            self.log('Estimated total time:        %10.3f seconds' %
                     (t * self.npairs0 / self.world.size))
            self.write_timing_information = False

        return e_n

    def calculate_exx_paw_correction(self):
        self.timer.start('PAW correction')
        self.devv = 0.0
        self.evc = 0.0
        self.ecc = 0.0
                         
        deg = 2 // self.wfs.nspins  # spin degeneracy
        for a, D_sp in self.dens.D_asp.items():
            setup = self.wfs.setups[a]
            for D_p in D_sp:
                D_ii = unpack2(D_p)
                ni = len(D_ii)

                for i1 in range(ni):
                    for i2 in range(ni):
                        A = 0.0
                        for i3 in range(ni):
                            p13 = packed_index(i1, i3, ni)
                            for i4 in range(ni):
                                p24 = packed_index(i2, i4, ni)
                                A += setup.M_pp[p13, p24] * D_ii[i3, i4]
                        self.devv -= D_ii[i1, i2] * A / deg

                self.evc -= np.dot(D_p, setup.X_p)
            self.ecc += setup.ExxC

        if not self.bandstructure:
            self.timer.stop('PAW correction')
            return

        Q = self.world.size // self.wfs.kd.comm.size
        self.exx_skn *= Q
        for kpt in self.wfs.kpt_u:
            for a, D_sp in self.dens.D_asp.items():
                setup = self.wfs.setups[a]
                for D_p in D_sp:
                    D_ii = unpack2(D_p)
                    ni = len(D_ii)
                    P_ni = kpt.P_ani[a]
                    for i1 in range(ni):
                        for i2 in range(ni):
                            A = 0.0
                            for i3 in range(ni):
                                p13 = packed_index(i1, i3, ni)
                                for i4 in range(ni):
                                    p24 = packed_index(i2, i4, ni)
                                    A += setup.M_pp[p13, p24] * D_ii[i3, i4]
                            self.exx_skn[kpt.s, kpt.k] -= \
                                (A * P_ni[:, i1].conj() * P_ni[:, i2]).real
                            p12 = packed_index(i1, i2, ni)
                            self.exx_skn[kpt.s, kpt.k] -= \
                                (P_ni[:, i1].conj() * setup.X_p[p12] *
                                 P_ni[:, i2]).real / self.wfs.nspins

        self.world.sum(self.exx_skn)
        self.exx_skn *= self.hybrid / Q
        self.timer.stop('PAW correction')
    
    def initialize_gaussian(self):
        """Calculate gaussian compensation charge and its potential.

        Used to decouple electrostatic interactions between
        periodically repeated images for molecular calculations.

        Charge containing one electron::

            (beta/pi)^(3/2)*exp(-beta*r^2),

        its Fourier transform::

            exp(-G^2/(4*beta)),

        and its potential::

            erf(beta^0.5*r)/r.
        """

        gd = self.wfs.gd

        # Set exponent of exp-function to -19 on the boundary:
        self.beta = 4 * 19 * (gd.icell_cv**2).sum(1).max()

        # Calculate gaussian:
        G_Gv = self.pd2.get_reciprocal_vectors()
        G2_G = self.pd2.G2_qG[0]
        C_v = gd.cell_cv.sum(0) / 2  # center of cell
        self.ngauss_G = np.exp(-1.0 / (4 * self.beta) * G2_G +
                                1j * np.dot(G_Gv, C_v)) / gd.dv

        # Calculate potential from gaussian:
        R_Rv = gd.get_grid_point_coordinates().transpose((1, 2, 3, 0))
        r_R = ((R_Rv - C_v)**2).sum(3)**0.5
        if (gd.N_c % 2 == 0).all():
            r_R[tuple(gd.N_c // 2)] = 1.0  # avoid dividing by zero
        v_R = erf(self.beta**0.5 * r_R) / r_R
        if (gd.N_c % 2 == 0).all():
            v_R[tuple(gd.N_c // 2)] = (4 * self.beta / pi)**0.5
        self.vgauss_G = self.pd2.fft(v_R)

        # Compare self-interaction to analytic result:
        assert abs(0.5 * self.pd2.integrate(self.ngauss_G, self.vgauss_G) -
                   (self.beta / 2 / pi)**0.5) < 1e-6


class KPoint:
    def __init__(self, wfs, nbands):
        """Helper class for parallelizing over k-points.

        Placeholder for wave functions, occupation numbers, eigenvalues,
        projections, spin index and global k-point index."""
        
        self.kd = wfs.kd
        self.ng_k = wfs.ng_k

        # Array large enough to hold wave functions from all k-points:
        self.psit_nG = wfs.pd.empty(nbands)

        self.requests = []

    def initialize(self, kpt):
        ng = self.ng_k[kpt.k]
        nbands = len(self.psit_nG)
        self.psit_nG[:, :ng] = kpt.psit_nG[:nbands]
        self.f_n = kpt.f_n / kpt.weight  # will be in the range [0,1]
        self.eps_n = kpt.eps_n
        self.P_ani = kpt.P_ani
        self.k = kpt.k
        self.s = kpt.s

        return self

    def next(self, wfs):
        """Create empty object.

        Data will be received from another process."""
        
        nbands = len(self.psit_nG)
        kpt = KPoint(wfs, nbands)

        # Allocate arrays for receiving:
        kpt.f_n = wfs.bd.empty()
        kpt.eps_n = wfs.bd.empty()

        # Total number of projector functions:
        I = sum([P_ni.shape[1] for P_ni in self.P_ani.values()])
        
        kpt.P_nI = np.empty((wfs.bd.nbands, I), wfs.dtype)

        kpt.P_ani = {}
        I1 = 0
        for a, P_ni in self.P_ani.items():
            I2 = I1 + P_ni.shape[1]
            kpt.P_ani[a] = kpt.P_nI[:, I1:I2]
            I1 = I2

        kpt.k = (self.k + 1) % self.kd.nibzkpts
        kpt.s = self.s
        
        return kpt
        
    def start_sending(self, rank):
        P_nI = np.hstack([P_ni for P_ni in self.P_ani.values()])
        P_nI = np.ascontiguousarray(P_nI)
        self.requests += [
            self.kd.comm.send(self.psit_nG, rank, block=False, tag=1),
            self.kd.comm.send(self.f_n, rank, block=False, tag=2),
            self.kd.comm.send(self.eps_n, rank, block=False, tag=3),
            self.kd.comm.send(P_nI, rank, block=False, tag=4)]
        
    def start_receiving(self, rank):
        self.requests += [
            self.kd.comm.receive(self.psit_nG, rank, block=False, tag=1),
            self.kd.comm.receive(self.f_n, rank, block=False, tag=2),
            self.kd.comm.receive(self.eps_n, rank, block=False, tag=3),
            self.kd.comm.receive(self.P_nI, rank, block=False, tag=4)]
    
    def wait(self):
        self.kd.comm.waitall(self.requests)
        self.requests = []



from ase.structure import molecule
from ase.parallel import parprint

from gpaw import GPAW
from gpaw.cluster import Cluster
from gpaw.analyse.hirshfeld import HirshfeldDensity, HirshfeldPartitioning
from gpaw.analyse.wignerseitz import WignerSeitz
from gpaw.test import equal
from gpaw.mpi import rank

h = 0.4
gpwname = 'H2O' + str(h) + '.gpw'

def run(lastres=[]):
    results = []

    # Hirshfeld ----------------------------------------

    if 1:

        hd = HirshfeldDensity(calc)

        # check for the number of electrons
        expected = [[None, 10],
                    [[0, 1, 2], 10],
                    [[1, 2], 2],
                    [[0], 8],
                    ]
        #expected = [[[0, 2], 9], ]
        #expected = [[None, 10], ]
        for gridrefinement in [1, 2, 4]:
            #Test for all gridrefinements for get_all_electron_density
            parprint('grid refinement', gridrefinement)
            for result in expected:
                indicees, result = result
                full, gd = hd.get_density(indicees, gridrefinement)
                parprint('indicees', indicees, end=': ') 
                parprint('result, expected:', gd.integrate(full), result)
                if gridrefinement < 4:
                    #The highest level of gridrefinement gets wrong electron numbers
                    equal(gd.integrate(full), result, 1.e-8)
                else:
                    equal(gd.integrate(full), result, 1.e-4)

        hp = HirshfeldPartitioning(calc)
        vr = hp.get_effective_volume_ratios()
        parprint('Hirshfeld:', vr)
        if len(lastres):
            equal(vr, lastres.pop(0), 1.e-10)
        results.append(vr)

    # Wigner-Seitz ----------------------------------------

    if 1:
        ws = WignerSeitz(calc.density.finegd, mol, calc)

        vr = ws.get_effective_volume_ratios()
        parprint('Wigner-Seitz:', vr)
        if len(lastres):
            equal(vr, lastres.pop(0), 1.e-10)
        results.append(vr)

    return results

mol = Cluster(molecule('H2O'))
mol.minimal_box(2.5, h=h)

# calculate
if 1:
    parprint('### fresh:')
    calc = GPAW(nbands=6,
                h = h, 
                txt=None)
if 1:
    calc.calculate(mol)
    calc.write(gpwname)
    lastres = run()

# load previous calculation
if 1:
    parprint('### reloaded:')
    calc = GPAW(gpwname, txt=None)
    mol = calc.get_atoms()
    run(lastres)

# periodic modulo test
parprint('### periodic:')
mol.set_pbc(True)
mol.translate(-mol[0].position)
mol.translate([-1.e-24, 0, 0])
calc.calculate(mol)
run()





from __future__ import print_function
from math import pi, cos, sin
from ase import Atom, Atoms
from gpaw import GPAW, PoissonSolver
from gpaw.test import equal, gen

# Generate setup for oxygen with half a core-hole:
gen('O', name='hch1s', corehole=(1, 0, 0.5))

if 1:
    a = 5.0
    d = 0.9575
    t = pi / 180 * 104.51
    H2O = Atoms([Atom('O', (0, 0, 0)),
                 Atom('H', (d, 0, 0)),
                 Atom('H', (d * cos(t), d * sin(t), 0))],
                cell=(a, a, a), pbc=False)
    H2O.center()
    calc = GPAW(nbands=10, h=0.2, setups={'O': 'hch1s'},
                poissonsolver=PoissonSolver(use_charge_center=True))
    H2O.set_calculator(calc)
    e = H2O.get_potential_energy()
    niter = calc.get_number_of_iterations()
    calc.write('h2o.gpw')
else:
    calc = GPAW('h2o.gpw',
                poissonsolver=PoissonSolver(use_charge_center=True))
    calc.initialize_positions()

from gpaw.xas import RecursionMethod

if 1:
    r = RecursionMethod(calc)
    r.run(400)
    r.write('h2o.pckl')
else:
    r = RecursionMethod(filename='h2o.pckl')

print(e, niter)
energy_tolerance = 0.0002
niter_tolerance = 0
equal(e, -17.9772, energy_tolerance)
